@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: net.c,v 1.37 2004/10/09 11:36:36 shirok Exp $
+ *  $Id: net.c,v 1.41 2005/10/13 08:14:13 shirok Exp $
  */
 
 #include "gauche/net.h"
@@ -237,7 +237,8 @@ ScmObj Scm_SocketListen(ScmSocket *sock, int backlog)
 ScmObj Scm_SocketAccept(ScmSocket *sock)
 {
     const char addrbuf[SCM_SOCKADDR_MAXLEN];
-    int newfd, addrlen = SCM_SOCKADDR_MAXLEN;
+    int newfd;
+    socklen_t addrlen = SCM_SOCKADDR_MAXLEN;
     ScmSocket *newsock;
     ScmClass *addrClass = Scm_ClassOf(SCM_OBJ(sock->address));
     
@@ -279,7 +280,8 @@ ScmObj Scm_SocketConnect(ScmSocket *sock, ScmSockAddr *addr)
 ScmObj Scm_SocketGetSockName(ScmSocket *sock)
 {
     const char addrbuf[SCM_SOCKADDR_MAXLEN];
-    int r, addrlen = SCM_SOCKADDR_MAXLEN;
+    int r;
+    socklen_t addrlen = SCM_SOCKADDR_MAXLEN;
 
     if (SOCKET_CLOSED(sock->fd)) {
         Scm_Error("attempt to get the name of a closed socket: %S", sock);
@@ -294,7 +296,8 @@ ScmObj Scm_SocketGetSockName(ScmSocket *sock)
 ScmObj Scm_SocketGetPeerName(ScmSocket *sock)
 {
     const char addrbuf[SCM_SOCKADDR_MAXLEN];
-    int r, addrlen = SCM_SOCKADDR_MAXLEN;
+    int r;
+    socklen_t addrlen = SCM_SOCKADDR_MAXLEN;
 
     if (SOCKET_CLOSED(sock->fd)) {
         Scm_Error("attempt to get the name of a closed socket: %S", sock);
@@ -308,13 +311,13 @@ ScmObj Scm_SocketGetPeerName(ScmSocket *sock)
 
 ScmObj Scm_SocketSend(ScmSocket *sock, ScmString *msg, int flags)
 {
-    int r;
+    int r; u_int size;
+    const char *cmsg;
     if (SOCKET_CLOSED(sock->fd)) {
         Scm_Error("attempt to send to a closed socket: %S", sock);
     }
-    SCM_SYSCALL(r, send(sock->fd,
-                        SCM_STRING_START(msg), SCM_STRING_SIZE(msg),
-                        flags));
+    cmsg = Scm_GetStringContent(msg, &size, NULL, NULL);
+    SCM_SYSCALL(r, send(sock->fd, cmsg, size, flags));
     if (r < 0) {
         Scm_SysError("send(2) failed");
     }
@@ -324,13 +327,13 @@ ScmObj Scm_SocketSend(ScmSocket *sock, ScmString *msg, int flags)
 ScmObj Scm_SocketSendTo(ScmSocket *sock, ScmString *msg, ScmSockAddr *to,
                         int flags)
 {
-    int r;
+    int r; u_int size;
+    const char *cmsg;
     if (SOCKET_CLOSED(sock->fd)) {
         Scm_Error("attempt to send to a closed socket: %S", sock);
     }
-    SCM_SYSCALL(r, sendto(sock->fd,
-                          SCM_STRING_START(msg), SCM_STRING_SIZE(msg),
-                          flags,
+    cmsg = Scm_GetStringContent(msg, &size, NULL, NULL);
+    SCM_SYSCALL(r, sendto(sock->fd, cmsg, size, flags,
                           &SCM_SOCKADDR(to)->addr, SCM_SOCKADDR(to)->addrlen));
     if (r < 0) {
         Scm_SysError("sendto(2) failed");
@@ -358,7 +361,7 @@ ScmObj Scm_SocketRecvFrom(ScmSocket *sock, int bytes, int flags)
     int r;
     char *buf;
     struct sockaddr from;
-    int fromlen = sizeof(from);
+    socklen_t fromlen = sizeof(from);
     if (SOCKET_CLOSED(sock->fd)) {
         Scm_Error("attempt to recv from a closed socket: %S", sock);
     }
@@ -384,8 +387,10 @@ ScmObj Scm_SocketSetOpt(ScmSocket *s, int level, int option, ScmObj value)
         Scm_Error("attempt to set a socket option of a closed socket: %S", s);
     }
     if (SCM_STRINGP(value)) {
-        SCM_SYSCALL(r, setsockopt(s->fd, level, option, SCM_STRING_START(value),
-                                    SCM_STRING_SIZE(value)));
+        u_int size;
+        const char *cvalue = Scm_GetStringContent(SCM_STRING(value), &size,
+                                                  NULL, NULL);
+        SCM_SYSCALL(r, setsockopt(s->fd, level, option, cvalue, size));
     } else if (SCM_INTP(value) || SCM_BIGNUMP(value)) {
         int v = Scm_GetInteger(value);
         SCM_SYSCALL(r, setsockopt(s->fd, level, option, &v, sizeof(int)));
@@ -399,18 +404,19 @@ ScmObj Scm_SocketSetOpt(ScmSocket *s, int level, int option, ScmObj value)
 ScmObj Scm_SocketGetOpt(ScmSocket *s, int level, int option, int rsize)
 {
     int r = 0;
+    socklen_t rrsize = rsize;
     if (SOCKET_CLOSED(s->fd)) {
         Scm_Error("attempt to get a socket option of a closed socket: %S", s);
     }
     if (rsize > 0) {
-        char *buf = SCM_NEW_ATOMIC2(char *, rsize);
-        SCM_SYSCALL(r, getsockopt(s->fd, level, option, buf, &rsize));
+        char *buf = SCM_NEW_ATOMIC2(char *, rrsize);
+        SCM_SYSCALL(r, getsockopt(s->fd, level, option, buf, &rrsize));
         if (r < 0) Scm_SysError("getsockopt failed");
-        return Scm_MakeString(buf, rsize, rsize, SCM_MAKSTR_INCOMPLETE);
+        return Scm_MakeString(buf, rrsize, rrsize, SCM_MAKSTR_INCOMPLETE);
     } else {
         int val;
-        rsize = sizeof(int);
-        SCM_SYSCALL(r, getsockopt(s->fd, level, option, &val, &rsize));
+        rrsize = sizeof(int);
+        SCM_SYSCALL(r, getsockopt(s->fd, level, option, &val, &rrsize));
         if (r < 0) Scm_SysError("getsockopt failed");
         return Scm_MakeInteger(val);
     }
@@ -447,13 +453,14 @@ static WSADATA wsaData;
 extern void Scm_Init_NetAddr(ScmModule *mod);
 extern void Scm_Init_NetDB(ScmModule *mod);
 extern void Scm_Init_netlib(ScmModule *mod);
+extern void Scm_Init_netaux(void);
 
 void Scm_Init_libnet(void)
 {
     ScmModule *mod;
 
     SCM_INIT_EXTENSION(net);
-    mod = SCM_MODULE(SCM_FIND_MODULE("gauche.net", TRUE));
+    mod = SCM_FIND_MODULE("gauche.net", SCM_FIND_MODULE_CREATE);
 #ifdef __MINGW32__
     /* NB: I'm supposed to call WSACleanup when application shuts down,
        or resource leak would happen, according to the Windows document.
@@ -482,57 +489,5 @@ void Scm_Init_libnet(void)
     Scm_Init_NetAddr(mod);
     Scm_Init_NetDB(mod);
     Scm_Init_netlib(mod);
-
-    /* Constants for socket option operation.
-       I define them here, instead of netlib.stub,  so that I can check
-       if the symbol is defined */
-#define DEFSYM(sym, val) \
-    SCM_DEFINE(mod, sym, Scm_MakeInteger(val))
-
-    DEFSYM("SOL_SOCKET", SOL_SOCKET);
-#ifdef SO_KEEPALIVE
-    DEFSYM("SO_KEEPALIVE", SO_KEEPALIVE);
-#endif
-#ifdef SO_OOBINLINE
-    DEFSYM("SO_OOBINLINE", SO_OOBINLINE);
-#endif
-#ifdef SO_REUSEADDR
-    DEFSYM("SO_REUSEADDR", SO_REUSEADDR);
-#endif
-#ifdef SO_TYPE
-    DEFSYM("SO_TYPE",      SO_TYPE);
-#endif
-#ifdef SO_BROADCAST
-    DEFSYM("SO_BROADCAST", SO_BROADCAST);
-#endif
-#ifdef SO_SNDBUF
-    DEFSYM("SO_SNDBUF",    SO_SNDBUF);
-#endif
-#ifdef SO_RCVBUF
-    DEFSYM("SO_RCVBUF",    SO_RCVBUF);
-#endif
-#ifdef SO_PRIORITY
-    DEFSYM("SO_PRIORITY",  SO_PRIORITY);
-#endif
-    DEFSYM("SO_ERROR",     SO_ERROR);
-
-#ifdef SOL_TCP
-    DEFSYM("SOL_TCP", SOL_TCP);
-#ifdef TCP_NODELAY
-    DEFSYM("TCP_NODELAY",  TCP_NODELAY);
-#endif
-#ifdef TCP_MAXSEG
-    DEFSYM("TCP_MAXSEG",   TCP_MAXSEG);
-#endif
-#ifdef TCP_CORK
-    DEFSYM("TCP_CORK",     TCP_CORK);
-#endif
-#endif /* SOL_TCP */
-
-#ifdef SOL_IP
-    DEFSYM("SOL_IP", SOL_IP);
-#ifdef IP_OPTIONS
-    DEFSYM("IP_OPTIONS",   IP_OPTIONS);
-#endif
-#endif /* SOL_IP */
+    Scm_Init_netaux();
 }
