@@ -1,7 +1,7 @@
 ;;;
 ;;; cgi.scm - CGI utility
 ;;;  
-;;;   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: cgi.scm,v 1.24 2004/11/29 09:06:23 shirok Exp $
+;;;  $Id: cgi.scm,v 1.28 2005/10/30 14:04:01 shirok Exp $
 ;;;
 
 ;; Surprisingly, there's no ``formal'' definition of CGI.
@@ -135,10 +135,9 @@
                 (member typesig
                         '(("application" "x-www-form-urlencoded")
                           ("multipart" "form-data"))))
-      (raise (condition
-              (<cgi-content-type-error>
-               (content-type type)
-               (message (format "Unsupported CONTENT_TYPE: ~a" type))))))
+      (errorf <cgi-content-type-error>
+              :content-type type
+              "Unsupported CONTENT_TYPE: ~a" type))
     (cond ((not method)  ;; interactive use.
            (if (sys-isatty (current-input-port))
              (begin
@@ -152,10 +151,9 @@
                          (if (string-null? line)
                            params
                            (cons line params))))))
-             (raise (condition
-                     (<cgi-request-method-error>
-                      (request-method #f)
-                      (message "REQUEST_METHOD not defined"))))))
+             (error <cgi-request-method-error>
+                    :request-method #f
+                    "REQUEST_METHOD not defined")))
           ((or (string-ci=? method "GET")
                (string-ci=? method "HEAD"))
            (or (get-meta "QUERY_STRING") ""))
@@ -169,10 +167,9 @@
                    (string-incomplete->complete (read-block len)))
                  (port->string (current-input-port)))))
           (else
-           (raise (condition
-                   (<cgi-request-method-error>
-                    (request-method method)
-                    (message (format "Unknown REQUEST_METHOD: ~a" method))))))
+           (errorf <cgi-request-method-error>
+                   :request-method method
+                   "Unknown REQUEST_METHOD: ~a" method))
           )))
 
 ;;----------------------------------------------------------------
@@ -290,9 +287,38 @@
      ((eq? action 'ignore) ignore-handler)
      (else action)))
 
+  ;; The value of content-disposition must be a properly quoted string
+  ;; according to RFC2183 and RFC2045.  However, IE sends a pathname including
+  ;; backslashes without quoting them.  As a compromise, we only consider
+  ;; backslash escape if the following character is either #\" or #\\.
+  ;; (This fix is provided by Tatsuya BIZENN).
+  (define (content-disposition-string input)
+    (read-char input)                   ; discard beginning DQUOTE
+    (let1 r (open-output-string :private? #t)
+      (define (finish) (get-output-string r))
+      (let loop ((c (read-char input)))
+        (cond ((eof-object? c) (finish)) ; tolerate missing closing DQUOTE
+              ((char=? c #\")  (finish)) ; discard ending DQUOTE
+              ((char=? c #\\)
+               (let1 c (read-char input)
+                 (cond ((eof-object? c) (finish)) ;; tolerate stray backslash
+                       (else
+                        (unless (char-set-contains? #[\\\"] c)
+                          (write-char #\\ r))
+                        (write-char c r)
+                        (loop (read-char input))))))
+              (else (write-char c r) (loop (read-char input)))))))
+
+  (define (parse-content-disposition field)
+    (if field
+      (rfc822-field->tokens field
+                            `((#[\"] . ,content-disposition-string)
+                              (,*rfc822-atext-chars* . ,rfc822-dot-atom)))
+      '()))
+
   (define (handle-part part-info inp)
     (let* ((cd   (part-ref part-info "content-disposition"))
-           (opts (if cd (rfc822-field->tokens cd) '()))
+           (opts (parse-content-disposition cd))
            (name (cond ((member "name=" opts) => cadr) (else #f)))
            (filename (cond ((member "filename=" opts) => cadr) (else #f)))
            )
@@ -368,13 +394,18 @@
                        (output-proc cgi-default-output)
                        (merge-cookies #f)
                        (part-handlers '()))
-    (with-error-handler
-     (lambda (e) (output-proc (on-error e)))
-     (lambda ()
-       (let1 params
-           (cgi-parse-parameters :merge-cookies merge-cookies
-                                 :part-handlers part-handlers)
-         (output-proc (proc params)))))
+    ;; we need to keep the output port, for the error handler may be
+    ;; called while the current output port is altered.
+    (let1 default-output (current-output-port)
+      (with-error-handler
+          (lambda (e)
+            (with-output-to-port default-output
+              (cut output-proc (on-error e))))
+        (lambda ()
+          (let1 params
+              (cgi-parse-parameters :merge-cookies merge-cookies
+                                    :part-handlers part-handlers)
+            (output-proc (proc params))))))
     ;; remove any temporary files
     (for-each sys-unlink (cgi-temporary-files))
     ))
@@ -389,8 +420,15 @@
                  (html:p (html-escape-string (slot-ref e 'message)))))))
 
 (define (cgi-default-output tree)
-  (write-tree tree (wrap-with-output-conversion
-                    (current-output-port)
-                    (cgi-output-character-encoding))))
+  (if (ces-equivalent? (gauche-character-encoding)
+                       (cgi-output-character-encoding))
+    (write-tree tree)
+    ;; NB: we avoid using wrap-with-output-conversion, for we want
+    ;; to make sure output is flushed each time in case cgi-main
+    ;; is used in a persistent process.
+    (with-output-conversion
+     (current-output-port)
+     (cut write-tree tree)
+     :encoding (cgi-output-character-encoding))))
 
 (provide "www/cgi")
