@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: code.c,v 1.6 2005/05/30 11:19:16 shirok Exp $
+ *  $Id: code.c,v 1.10 2005/10/13 08:14:13 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -206,7 +206,7 @@ void Scm_CompiledCodeDump(ScmCompiledCode *cc)
                 Scm_Puts(SCM_STRING(s), SCM_CUROUT);
                 Scm_Putc('\n', SCM_CUROUT);
             } else {
-                int len = SCM_STRING_SIZE(s);
+                int len = SCM_STRING_BODY_SIZE(SCM_STRING_BODY(s));
                 ScmObj srcinfo = Scm_Assq(SCM_SYM_SOURCE_INFO, info);
                 ScmObj bindinfo = Scm_Assq(SCM_SYM_BIND_INFO, info);
                 Scm_Puts(SCM_STRING(s), SCM_CUROUT);
@@ -256,6 +256,8 @@ typedef struct cc_builder_rec {
     ScmObj constants;           /* list of constants */
     int currentIndex;
     ScmWord currentInsn;        /* buffer for instruction combining. */
+    int    currentArg0;         /* ditto */
+    int    currentArg1;         /* ditto */
     ScmObj currentOperand;      /* ditto */
     ScmObj currentInfo;         /* ditto */
     ScmObj labelDefs;           /* alist of (name . offset) */
@@ -550,6 +552,33 @@ void Scm_CompiledCodeFinishBuilder(ScmCompiledCode *cc, int maxstack)
  * since this routine is the most frequently called one during compilation.
  */
 
+/* The plan is to use STN generated from vminsn.scm for instruction
+   combination, but we haven't got it working yet. */
+#if 0
+/* The state transition table */
+struct stn_arc {
+    int input;                  /* input insn, or -1 for wildcard */
+    int action;                 /* NEXT, RESET, KEEPn */
+    int operand;                /* emitting insn / next state */
+};
+
+/* State transition actions */
+enum {
+    NEXT,
+    EMIT,
+    KEEP
+};
+
+/* Include STN generated from vminsn.scm */
+static struct stn_arc stn[] = {
+#define STATE_TABLE
+#include "vminsn.c"
+#undef STATE_TABLE
+};
+#endif /*0*/
+/* The following is the legacy code (manually tweaked automaton).
+   Once we get STN working it will go away. */
+
 /* some abbreviations for better readability */
 
 #define INSN(x)         SCM_VM_INSN(x)
@@ -599,51 +628,51 @@ void Scm_CompiledCodeEmit(ScmCompiledCode *cc,
     cc_builder *b;
     CC_BUILDER_GET(b, cc);
 
+    if (SCM_VM_COMPILER_FLAG_IS_SET(Scm_VM(), SCM_COMPILE_NOCOMBINE)) {
+        goto def;
+    }
+
+
     switch (code) {
     case SCM_VM_LREF:
     {
-        static const int lrefs[] = {
-            SCM_VM_LREF0, SCM_VM_LREF1, SCM_VM_LREF2,
-            SCM_VM_LREF3, SCM_VM_LREF4,
-            SCM_VM_LREF10, SCM_VM_LREF11, SCM_VM_LREF12,
-            SCM_VM_LREF13, SCM_VM_LREF14
+        static const int lrefs[4][4] = {
+            { SCM_VM_LREF0,  SCM_VM_LREF1,  SCM_VM_LREF2,  SCM_VM_LREF3 },
+            { SCM_VM_LREF10, SCM_VM_LREF11, SCM_VM_LREF12, -1 },
+            { SCM_VM_LREF20, SCM_VM_LREF21, -1, -1 },
+            { SCM_VM_LREF30, -1, -1, -1 }
         };
-        if (arg0 <= 1 && arg1 <= 4) {
-            PUT(INSN(lrefs[arg0*5+arg1]), SCM_FALSE);
-        } else {
-            PUT(INSN2(SCM_VM_LREF, arg0, arg1), SCM_FALSE);
+        if (arg0 < 4 && arg1 < 4) {
+            int insn = lrefs[arg0][arg1];
+            if (insn >= 0) {
+                PUT(INSN(insn), SCM_FALSE);
+                break;
+            }
         }
+        PUT(INSN2(SCM_VM_LREF, arg0, arg1), SCM_FALSE);
         break;
     }
     
-    case SCM_VM_LSET:
-    {
-        static const int lsets[] = {
-            SCM_VM_LSET0, SCM_VM_LSET1, SCM_VM_LSET2,
-            SCM_VM_LSET3, SCM_VM_LSET4,
-        };
-        if (arg0 == 0 && arg1 <= 4) {
-            PUT(INSN(lsets[arg1]), SCM_FALSE);
-        } else {
-            PUT(INSN2(SCM_VM_LSET, arg0, arg1), SCM_FALSE);
-        }
-        break;
-    }
-
     case SCM_VM_PUSH:
     {
         if (EMPTYP(b)) goto def;
         switch (CODE(b->currentInsn)) {
-        case SCM_VM_LREF0:  SUB(INSN(SCM_VM_LREF0_PUSH)); break;
-        case SCM_VM_LREF1:  SUB(INSN(SCM_VM_LREF1_PUSH)); break;
-        case SCM_VM_LREF2:  SUB(INSN(SCM_VM_LREF2_PUSH)); break;
-        case SCM_VM_LREF3:  SUB(INSN(SCM_VM_LREF3_PUSH)); break;
-        case SCM_VM_LREF4:  SUB(INSN(SCM_VM_LREF4_PUSH)); break;
+        case SCM_VM_LREF0: SUB(INSN(SCM_VM_LREF0_PUSH)); break;
+        case SCM_VM_LREF1: SUB(INSN(SCM_VM_LREF1_PUSH)); break;
+        case SCM_VM_LREF2: SUB(INSN(SCM_VM_LREF2_PUSH)); break;
+        case SCM_VM_LREF3: SUB(INSN(SCM_VM_LREF3_PUSH)); break;
         case SCM_VM_LREF10: SUB(INSN(SCM_VM_LREF10_PUSH)); break;
         case SCM_VM_LREF11: SUB(INSN(SCM_VM_LREF11_PUSH)); break;
         case SCM_VM_LREF12: SUB(INSN(SCM_VM_LREF12_PUSH)); break;
+        case SCM_VM_LREF20: SUB(INSN(SCM_VM_LREF20_PUSH)); break;
+        case SCM_VM_LREF21: SUB(INSN(SCM_VM_LREF21_PUSH)); break;
+        case SCM_VM_LREF30: SUB(INSN(SCM_VM_LREF30_PUSH)); break;
+
+        /* obsoleted */
+        case SCM_VM_LREF4:  SUB(INSN(SCM_VM_LREF4_PUSH)); break;
         case SCM_VM_LREF13: SUB(INSN(SCM_VM_LREF13_PUSH)); break;
         case SCM_VM_LREF14: SUB(INSN(SCM_VM_LREF14_PUSH)); break;
+
         case SCM_VM_LREF:   SUB(INSN2(SCM_VM_LREF_PUSH,
                                       IARG0(b->currentInsn),
                                       IARG1(b->currentInsn))); break;
@@ -691,6 +720,8 @@ void Scm_CompiledCodeEmit(ScmCompiledCode *cc,
             SUB(INSN1(SCM_VM_GREF_CALL, arg0)); break;
         case SCM_VM_PUSH_GREF:
             SUB(INSN1(SCM_VM_PUSH_GREF_CALL, arg0)); break;
+        case SCM_VM_LREF0_PUSH_GREF:
+            SUB(INSN1(SCM_VM_LREF0_PUSH_GREF_CALL, arg0)); break;
         default:
             PUT(INSN1(SCM_VM_CALL, arg0), SCM_FALSE);
         }
@@ -705,6 +736,8 @@ void Scm_CompiledCodeEmit(ScmCompiledCode *cc,
             SUB(INSN1(SCM_VM_GREF_TAIL_CALL, arg0)); break;
         case SCM_VM_PUSH_GREF:
             SUB(INSN1(SCM_VM_PUSH_GREF_TAIL_CALL, arg0)); break;
+        case SCM_VM_LREF0_PUSH_GREF:
+            SUB(INSN1(SCM_VM_LREF0_PUSH_GREF_TAIL_CALL, arg0)); break;
         default:
             PUT(INSN1(SCM_VM_TAIL_CALL, arg0), SCM_FALSE);
         }
@@ -723,11 +756,16 @@ void Scm_CompiledCodeEmit(ScmCompiledCode *cc,
 
     case SCM_VM_GREF:
     {
-        if (!EMPTYP(b) && CODE(b->currentInsn) == SCM_VM_PUSH) {
-            SUBO(INSN1(SCM_VM_PUSH_GREF, arg0), operand);
-        } else {
-            PUT(INSN1(SCM_VM_GREF, arg0), operand);
+        if (!EMPTYP(b)) {
+            if (CODE(b->currentInsn) == SCM_VM_PUSH) {
+                SUBO(INSN1(SCM_VM_PUSH_GREF, arg0), operand);
+                break;
+            } else if (CODE(b->currentInsn) == SCM_VM_LREF0_PUSH) {
+                SUBO(INSN1(SCM_VM_LREF0_PUSH_GREF, arg0), operand);
+                break;
+            }
         }
+        PUT(INSN1(SCM_VM_GREF, arg0), operand);
         break;
     }
 
@@ -777,6 +815,23 @@ void Scm_CompiledCodeEmit(ScmCompiledCode *cc,
         }
         break;
     }
+
+#if 0
+    case SCM_VM_NUMADDI:
+    {
+        if (EMPTYP(b)) goto def;
+        switch (CODE(b->currentInsn)) {
+        case SCM_VM_LREF0:  SUB(INSN1(SCM_VM_LREF0_NUMADDI, arg0)); break;
+        case SCM_VM_LREF1:  SUB(INSN1(SCM_VM_LREF1_NUMADDI, arg0)); break;
+        case SCM_VM_LREF2:  SUB(INSN1(SCM_VM_LREF2_NUMADDI, arg0)); break;
+        case SCM_VM_LREF3:  SUB(INSN1(SCM_VM_LREF3_NUMADDI, arg0)); break;
+        case SCM_VM_LREF4:  SUB(INSN1(SCM_VM_LREF4_NUMADDI, arg0)); break;
+        default:
+            PUT(INSN1(SCM_VM_NUMADDI, arg0), SCM_FALSE);
+        }
+        break;
+    }
+#endif
     
     default:;
     def:
