@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: load.c,v 1.98 2005/05/17 04:33:09 shirok Exp $
+ *  $Id: load.c,v 1.103 2005/10/13 08:14:13 shirok Exp $
  */
 
 #include <stdlib.h>
@@ -48,6 +48,9 @@
 
 #define LOAD_SUFFIX ".scm"      /* default load suffix */
 
+/* for tuning.  define this to display load timing info */
+#undef SHOW_LOAD_TIMINGS
+
 /*
  * Load file.
  */
@@ -121,6 +124,15 @@ static ScmObj load_after(ScmObj *args, int nargs, void *data)
 {
     struct load_packet *p = (struct load_packet *)data;
     ScmVM *vm = Scm_VM();
+
+#ifdef SHOW_LOAD_TIMINGS
+    struct timeval t0;
+    gettimeofday(&t0, NULL);
+    fprintf(stdout, "%10u)\n",
+            t0.tv_sec*1000000+t0.tv_usec,
+            Scm_GetStringConst(SCM_STRING(Scm_PortName(p->port))));
+#endif /*SHOW_LOAD_TIMINGS*/
+
     Scm_ClosePort(p->port);
     PORT_UNLOCK(p->port);
     Scm_SelectModule(p->prev_module);
@@ -269,8 +281,8 @@ static ScmObj try_suffixes(ScmObj base, ScmObj suffixes)
 ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
                     ScmObj suffixes, int flags)
 {
-    int size = SCM_STRING_LENGTH(filename);
-    const char *ptr = SCM_STRING_START(filename);
+    u_int size;
+    const char *ptr = Scm_GetStringContent(filename, &size, NULL, NULL);
     int use_load_paths = TRUE;
     ScmObj file = SCM_OBJ(filename), fpath = SCM_FALSE;
 
@@ -346,6 +358,15 @@ ScmObj Scm_VMLoad(ScmString *filename, ScmObj load_paths,
     truename = Scm_FindFile(filename, &load_paths, suffixes, flags);
     if (SCM_FALSEP(truename)) return SCM_FALSE;
 
+#ifdef SHOW_LOAD_TIMINGS
+    {
+        struct timeval t0;
+        gettimeofday(&t0, NULL);
+        fprintf(stdout, "(\"%s\" %10u\n",
+                Scm_GetStringConst(SCM_STRING(truename)),
+                t0.tv_sec*1000000+t0.tv_usec);
+    }
+#endif /*SHOW_LOAD_TIMINGS*/
     if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
         int len = Scm_Length(vm->load_history);
         SCM_PUTZ(";;", 2, SCM_CURERR);
@@ -359,11 +380,9 @@ ScmObj Scm_VMLoad(ScmString *filename, ScmObj load_paths,
         if (errorp) Scm_Error("file %S exists, but couldn't open.", truename);
         else        return SCM_FALSE;
     }
-
     if (!ignore_coding) {
         port = Scm_MakeCodingAwarePort(SCM_PORT(port));
     }
-    
     return Scm_VMLoadFromPort(SCM_PORT(port), load_paths, env, flags);
 }
 
@@ -967,6 +986,8 @@ void Scm_DefineAutoload(ScmModule *where,
 ScmObj Scm_LoadAutoload(ScmAutoload *adata)
 {
     int error = FALSE;
+    ScmModule *prev_module;
+    ScmVM *vm = Scm_VM();
     
     /* check if some other thread already loaded this before attempt to lock */
     if (adata->loaded) {
@@ -978,14 +999,14 @@ ScmObj Scm_LoadAutoload(ScmAutoload *adata)
     do {
         if (adata->loaded) break;
         if (adata->locker == NULL) {
-            adata->locker = Scm_VM(); /* take me */
-        } else if (adata->locker == Scm_VM()) {
+            adata->locker = vm;
+        } else if (adata->locker == vm) {
             /* bad circular dependency */
             error = TRUE;
         } else if (adata->locker->state == SCM_VM_TERMINATED) {
             /* the loading thread have died prematurely.
                let's take over the task. */
-            adata->locker = Scm_VM();
+            adata->locker = vm;
         } else {
             (void)SCM_INTERNAL_COND_WAIT(adata->cv, adata->mutex);
             continue;
@@ -1004,15 +1025,19 @@ ScmObj Scm_LoadAutoload(ScmAutoload *adata)
                   adata->module, adata->name);
     }
 
+    prev_module = vm->module;
     SCM_UNWIND_PROTECT {
+        vm->module = adata->module;
         Scm_Require(SCM_OBJ(adata->path));
+        vm->module = prev_module;
     
         if (adata->import_from) {
             /* autoloaded file defines import_from module.  we need to
                import the binding individually. */
-            ScmObj m = Scm_FindModule(adata->import_from, FALSE);
+            ScmModule *m = Scm_FindModule(adata->import_from,
+                                          SCM_FIND_MODULE_QUIET);
             ScmGloc *f, *g;
-            if (!SCM_MODULEP(m)) {
+            if (m == NULL) {
                 Scm_Error("Trying to autoload module %S from file %S, but the file doesn't define such a module",
                           adata->import_from, adata->path);
             }
@@ -1039,6 +1064,7 @@ ScmObj Scm_LoadAutoload(ScmAutoload *adata)
         }
     } SCM_WHEN_ERROR {
         adata->locker = NULL;
+        vm->module = prev_module;
         SCM_INTERNAL_COND_SIGNAL(adata->cv);
         SCM_NEXT_HANDLER;
     } SCM_END_PROTECT;
