@@ -1,7 +1,7 @@
 /*
  * module.c - module implementation
  *
- *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: module.c,v 1.52 2005/04/12 01:42:27 shirok Exp $
+ *  $Id: module.c,v 1.60 2005/10/13 08:14:13 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -62,8 +62,8 @@
  * to use a single global lock (modules.mutex), based on the following
  * observations:
  *
- *  - Profiling shows mutex_lock takes around 10% of program loading
- *    phase.
+ *  - Profiling showed mutex_lock was taking around 10% of program loading
+ *    phase in the previous version.
  *
  *  - Module operations almost always occur during program loading and
  *    interactive session.  Having giant lock for module operations won't
@@ -112,9 +112,10 @@ static void init_module(ScmModule *m, ScmSymbol *name)
 {
     m->name = name;
     m->imported = m->exported = SCM_NIL;
+    m->exportAll = FALSE;
     m->parents = defaultParents;
     m->mpl = Scm_Cons(SCM_OBJ(m), defaultMpl);
-    m->table = SCM_HASHTABLE(Scm_MakeHashTable(SCM_HASH_ADDRESS, NULL, 0));
+    m->table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
 }
 
 /* Internal */
@@ -197,15 +198,23 @@ ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
 
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
 
-    /* fist, search from the specified module */
+    /* first, search from the specified module.
+       NB: we directly check gloc->value instead of calling
+       SCM_GLOC_GET, since this check is merely to eliminate
+       the GLOC inserted by export. */
     e = Scm_HashTableGet(m->table, SCM_OBJ(symbol));
-    if (e) { gloc = SCM_GLOC(e->value); goto found; }
+    if (e) {
+        gloc = SCM_GLOC(e->value);
+        if (!SCM_UNBOUNDP(gloc->value)) goto found;
+    }
     
     if (!stay_in_module) {
         /* Next, search from imported modules */
         SCM_FOR_EACH(p, module->imported) {
             SCM_ASSERT(SCM_MODULEP(SCM_CAR(p)));
             SCM_FOR_EACH(mp, SCM_MODULE(SCM_CAR(p))->mpl) {
+                ScmGloc *g;
+                
                 SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
                 
                 for (i=0; i<num_searched; i++) {
@@ -219,10 +228,10 @@ ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
                 
                 m = SCM_MODULE(SCM_CAR(mp));
                 e = Scm_HashTableGet(m->table, SCM_OBJ(symbol));
-                if (e &&
-                    (SCM_TRUEP(m->exported)
-                     || !SCM_FALSEP(Scm_Memq(SCM_OBJ(symbol), m->exported)))) {
-                    gloc = SCM_GLOC(e->value);
+                /* see above comment about the check of gloc->value */
+                if (e && (g = SCM_GLOC(e->value))->exported
+                    && !SCM_UNBOUNDP(g->value)) {
+                    gloc = g;
                     goto found;
                 }
 
@@ -257,6 +266,7 @@ ScmObj Scm_SymbolValue(ScmModule *module, ScmSymbol *symbol)
 
 /*
  * Definition.
+ *  TODO: consolidate the common code between Scm_Define and Scm_DefineConst.
  */
 ScmObj Scm_Define(ScmModule *module, ScmSymbol *symbol, ScmObj value)
 {
@@ -277,6 +287,11 @@ ScmObj Scm_Define(ScmModule *module, ScmSymbol *symbol, ScmObj value)
         g = SCM_GLOC(Scm_MakeGloc(symbol, module));
         SCM_GLOC_SET(g, value);
         Scm_HashTablePut(module->table, SCM_OBJ(symbol), SCM_OBJ(g));
+        /* If module is marked 'export-all', export this binding by default */
+        if (module->exportAll) {
+            g->exported = TRUE;
+            module->exported = Scm_Cons(SCM_OBJ(g->name), module->exported);
+        }
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     
@@ -308,6 +323,11 @@ ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
         g = SCM_GLOC(Scm_MakeConstGloc(symbol, module));
         g->value = value;
         Scm_HashTablePut(module->table, SCM_OBJ(symbol), SCM_OBJ(g));
+        /* If module is marked 'export-all', export this binding by default */
+        if (module->exportAll) {
+            g->exported = TRUE;
+            module->exported = Scm_Cons(SCM_OBJ(g->name), module->exported);
+        }
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
 
@@ -319,7 +339,8 @@ ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
 
 ScmObj Scm_ImportModules(ScmModule *module, ScmObj list)
 {
-    ScmObj lp, mod;
+    ScmObj lp;
+    ScmModule *mod;
     ScmSymbol *name = NULL;
     SCM_FOR_EACH(lp, list) {
         if (SCM_SYMBOLP(SCM_CAR(lp))) {
@@ -329,11 +350,11 @@ ScmObj Scm_ImportModules(ScmModule *module, ScmObj list)
         } else {
             Scm_Error("module name required, but got %S", SCM_CAR(lp));
         }
-        mod = Scm_FindModule(name, FALSE);
-        if (!SCM_MODULEP(mod)) Scm_Error("no such module: %S", SCM_CAR(lp));
+        mod = Scm_FindModule(name, 0);
         (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
         module->imported =
-            Scm_Cons(mod, Scm_DeleteX(mod, module->imported, SCM_CMP_EQ));
+            Scm_Cons(SCM_OBJ(mod),
+                     Scm_DeleteX(SCM_OBJ(mod), module->imported, SCM_CMP_EQ));
         (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     }
     return module->imported;
@@ -343,21 +364,42 @@ ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
 {
     ScmObj lp, syms, badsym = SCM_FALSE;
     int error = FALSE;
+    ScmSymbol *s;
+    ScmHashEntry *e;
+    ScmGloc *g;
 
+    /* We used to do something like
+     *  (set! (module-exports module)
+     *        (delete-duplicates (union (module-exports module) list)))
+     * This is slow when we export lots of symbols.  As of 0.8.6,
+     * each GLOC has exported flag, so we can check whether a binding
+     * is exported or not in O(1).   Module-exports list is kept
+     * for backward compatibility.
+     */
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     syms = module->exported;
-    if (!SCM_TRUEP(syms)) {
-        SCM_FOR_EACH(lp, list) {
-            if (!SCM_SYMBOLP(SCM_CAR(lp))) {
-                error = TRUE;
-                badsym = SCM_CAR(lp);
-                break;
+    SCM_FOR_EACH(lp, list) {
+        if (!SCM_SYMBOLP(SCM_CAR(lp))) {
+            error = TRUE;
+            badsym = SCM_CAR(lp);
+            break;
+        }
+        s = SCM_SYMBOL(SCM_CAR(lp));
+        e = Scm_HashTableAdd(module->table, SCM_OBJ(s), SCM_UNBOUND);
+        if (SCM_GLOCP(e->value)) {
+            g = SCM_GLOC(e->value);
+            if (!g->exported) {
+                syms = Scm_Cons(SCM_OBJ(s), syms);
+                g->exported = TRUE;
             }
-            if (SCM_FALSEP(Scm_Memq(SCM_CAR(lp), syms)))
-                syms = Scm_Cons(SCM_CAR(lp), syms);
+        } else {
+            g = SCM_GLOC(Scm_MakeGloc(s, module));
+            g->exported = TRUE;
+            e->value = SCM_OBJ(g);
+            syms = Scm_Cons(SCM_OBJ(s), syms);
         }
-        if (!error) module->exported = syms;
     }
+    if (!error) module->exported = syms;
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     if (error) Scm_Error("symbol required, but got %S", badsym);
     return syms;
@@ -365,8 +407,26 @@ ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
 
 ScmObj Scm_ExportAll(ScmModule *module)
 {
+    ScmHashIter iter;
+    ScmHashEntry *e;
+    
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
-    module->exported = SCM_TRUE;
+    if (!module->exportAll) {
+        /* Mark the module 'export-all' so that the new bindings would get
+           exported mark by default. */
+        module->exportAll = TRUE;
+        
+        /* Scan the module and mark all existing bindings as exported. */
+        Scm_HashIterInit(module->table, &iter);
+        while ((e = Scm_HashIterNext(&iter)) != NULL) {
+            ScmGloc *g = SCM_GLOC(e->value);
+            if (!g->exported) {
+                g->exported = TRUE;
+                module->exported =
+                    Scm_Cons(SCM_OBJ(g->name), module->exported);
+            }
+        }
+    }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     return SCM_OBJ(module);
 }
@@ -405,18 +465,27 @@ ScmObj Scm_ExtendModule(ScmModule *module, ScmObj supers)
  * Finding modules
  */
 
-ScmObj Scm_FindModule(ScmSymbol *name, int createp)
+ScmModule *Scm_FindModule(ScmSymbol *name, int flags)
 {
     ScmModule *m;
     int created;
 
-    if (createp) {
+    if (flags & SCM_FIND_MODULE_CREATE) {
         m = lookup_module_create(name, &created);
+        SCM_ASSERT(m != NULL);
+        return m;
     } else {
         m = lookup_module(name);
+        if (m == NULL) {
+            if (flags & SCM_FIND_MODULE_QUIET) {
+                return NULL;
+            } else {
+                Scm_Error("no such module: %S", name);
+            }
+        } else {
+            return m;
+        }
     }
-    if (m) return SCM_OBJ(m);
-    else return SCM_FALSE;
 }
 
 ScmObj Scm_AllModules(void)
@@ -453,24 +522,26 @@ void Scm_SelectModule(ScmModule *mod)
 
 ScmObj Scm_ModuleNameToPath(ScmSymbol *name)
 {
-    char *buf = SCM_NEW_ATOMIC2(char *, SCM_STRING_SIZE(name->name)+1);
-    char *p = buf, *e = buf + SCM_STRING_SIZE(name->name);
-    memcpy(buf, SCM_STRING_START(name->name), SCM_STRING_SIZE(name->name));
+    const ScmStringBody *b = SCM_STRING_BODY(SCM_SYMBOL_NAME(name));
+    char *buf = SCM_NEW_ATOMIC2(char *, SCM_STRING_BODY_SIZE(b)+1);
+    char *p = buf, *e = buf + SCM_STRING_BODY_SIZE(b);
+    memcpy(buf, SCM_STRING_BODY_START(b), SCM_STRING_BODY_SIZE(b));
     while (p < e) {
         int n = SCM_CHAR_NFOLLOWS(*p);
         if (*p == '.') *p++ = '/';
         else p += n+1;
     }
     *e = '\0';
-    return Scm_MakeString(buf, SCM_STRING_SIZE(name->name),
-                          SCM_STRING_LENGTH(name->name), 0);
+    return Scm_MakeString(buf, SCM_STRING_BODY_SIZE(b),
+                          SCM_STRING_BODY_LENGTH(b), 0);
 }
 
 ScmObj Scm_PathToModuleName(ScmString *path)
 {
-    char *buf = SCM_NEW_ATOMIC2(char *, SCM_STRING_SIZE(path)+1);
-    char *p = buf, *e = buf + SCM_STRING_SIZE(path);
-    memcpy(buf, SCM_STRING_START(path), SCM_STRING_SIZE(path));
+    const ScmStringBody *b = SCM_STRING_BODY(path);
+    char *buf = SCM_NEW_ATOMIC2(char *, SCM_STRING_BODY_SIZE(b)+1);
+    char *p = buf, *e = buf + SCM_STRING_BODY_SIZE(b);
+    memcpy(buf, SCM_STRING_BODY_START(b), SCM_STRING_BODY_SIZE(b));
     while (p < e) {
         int n = SCM_CHAR_NFOLLOWS(*p);
         if (*p == '/') *p++ = '.';
@@ -531,7 +602,7 @@ void Scm__InitModule(void)
     ScmObj mpl = SCM_NIL;
 
     (void)SCM_INTERNAL_MUTEX_INIT(modules.mutex);
-    modules.table = SCM_HASHTABLE(Scm_MakeHashTable(SCM_HASH_ADDRESS, NULL, 64));
+    modules.table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 64));
 
     /* standard module chain */
     INIT_MOD(nullModule, SCM_SYM_NULL, mpl);
