@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: number.c,v 1.118 2005/04/12 01:42:27 shirok Exp $
+ *  $Id: number.c,v 1.123 2005/10/13 08:14:13 shirok Exp $
  */
 
 #include <math.h>
@@ -139,7 +139,7 @@ ScmObj Scm_MakeFlonum(double d)
 
 ScmObj Scm_MakeFlonumToNumber(double d, int exact)
 {
-    if (exact) {
+    if (exact && !SCM_IS_INF(d)) {
         /* see if d can be demoted to integer */
         double i, f;
         f = modf(d, &i);
@@ -762,21 +762,17 @@ ScmObj Scm_Reciprocal(ScmObj obj)
 {
     if (SCM_INTP(obj)) {
         long val = SCM_INT_VALUE(obj);
-        if (val == 0) Scm_Error("divide by zero");
         obj = Scm_MakeFlonum(1.0/(double)val);
     } else if (SCM_BIGNUMP(obj)) {
         double val = Scm_BignumToDouble(SCM_BIGNUM(obj));
-        if (val == 0.0) Scm_Error("divide by zero");
         obj = Scm_MakeFlonum(1.0/val);
     } else if (SCM_FLONUMP(obj)) {
         double val = SCM_FLONUM_VALUE(obj);
-        if (val == 0.0) Scm_Error("divide by zero");
         obj = Scm_MakeFlonum(1.0/val);
     } else if (SCM_COMPLEXP(obj)) {
         double r = SCM_COMPLEX_REAL(obj), r1;
         double i = SCM_COMPLEX_IMAG(obj), i1;
         double d;
-        if (r == 0.0 && i == 0.0) Scm_Error("divide by zero");
         d = r*r + i*i;
         r1 = r/d;
         i1 = -i/d;
@@ -1266,7 +1262,6 @@ ScmObj Scm_Divide(ScmObj arg0, ScmObj arg1, ScmObj args)
                                     Scm_MakeFlonumToNumber(result_real, exact),
                                     arg1, args);
             }
-            if (div_real == 0) Scm_Error("divide by zero");
             result_real /= div_real;
             if (SCM_NULLP(args))
                 return Scm_MakeFlonumToNumber(result_real, exact);
@@ -1297,7 +1292,6 @@ ScmObj Scm_Divide(ScmObj arg0, ScmObj arg1, ScmObj args)
                                     arg1, args);
             }
             d = div_real*div_real + div_imag*div_imag;
-            if (d == 0.0) Scm_Error("divide by zero");
             r = (result_real*div_real + result_imag*div_imag)/d;
             i = (result_imag*div_real - result_real*div_imag)/d;
             result_real = r;
@@ -1387,7 +1381,7 @@ ScmObj Scm_Quotient(ScmObj x, ScmObj y, ScmObj *rem)
         goto BADARG;
     }
   DIVBYZERO:
-    Scm_Error("divide by zero");
+    Scm_Error("attempt to calculate a quotient by zero");
   BADARGY:
     x = y;
   BADARG:
@@ -1493,7 +1487,7 @@ ScmObj Scm_Modulo(ScmObj x, ScmObj y, int remp)
         goto BADARG;
     }
   DIVBYZERO:
-    Scm_Error("divide by zero");
+    Scm_Error("attempt to take a modulo or remainder by zero");
   BADARGY:
     x = y;
   BADARG:
@@ -1508,7 +1502,8 @@ ScmObj Scm_Modulo(ScmObj x, ScmObj y, int remp)
 /* Integer power of 10.  It is extensively used during string->number
    and number->string operations.
    IEXPT10_TABLESIZ is ceil(-log10(ldexp(1.0, -1022-52))) + 2 */
-#define IEXPT10_TABLESIZ  326
+/* NB: actually we need more margin here to handle denormalized numbers. */
+#define IEXPT10_TABLESIZ  341
 static ScmObj iexpt10_n[IEXPT10_TABLESIZ] = { NULL };
 static int    iexpt10_initialized = FALSE;
 
@@ -1931,15 +1926,25 @@ static inline int numcmp3(ScmObj x, ScmObj d, ScmObj y)
 
 static void double_print(char *buf, int buflen, double val, int plus_sign)
 {
-    if (val < 0.0) *buf++ = '-', buflen--;
-    else if (plus_sign) *buf++ = '+', buflen--;
-    if (SCM_IS_INF(val)) {
-        strcpy(buf, "#<inf>");
+    /* Handle a few special cases first.
+       The notation of infinity is provisional; see how srfi-70 becomes. */
+    if (val == 0.0) {
+        if (plus_sign) strcpy(buf, "+0.0");
+        else strcpy(buf, "0.0");
+        return;
+    } else if (SCM_IS_INF(val)) {
+        if (val < 0.0) strcpy(buf, "#i-1/0");
+        else if (plus_sign) strcpy(buf, "#i+1/0");
+        else strcpy(buf, "#i1/0");
+        return;
     } else if (SCM_IS_NAN(val)) {
         strcpy(buf, "#<nan>");
-    } else if (val == 0.0) {
-        strcpy(buf, "0.0");
-    } else {
+        return;
+    }
+    
+    if (val < 0.0) *buf++ = '-', buflen--;
+    else if (plus_sign) *buf++ = '+', buflen--;
+    {
         /* variable names follows Burger&Dybvig paper. mp, mm for m+, m-.
            note that m+ == m- for most cases, and m+ == 2*m- for the rest.
            so we calculate m+ from m- for each iteration, using the flag
@@ -2448,8 +2453,11 @@ static ScmObj read_real(const char **strp, int *lenp,
             if (SCM_FALSEP(denom)) return SCM_FALSE;
             if (denom == SCM_MAKE_INT(0)) {
                 if (lensave > *lenp) {
-                    return numread_error("(zero in denominator of rational number)",
-                                         ctx);
+                    if (minusp) {
+                        return Scm_MakeFlonum(-1.0/0.0);
+                    } else {
+                        return Scm_MakeFlonum(1.0/0.0);
+                    }
                 } else {
                     return SCM_FALSE;
                 }
@@ -2675,12 +2683,13 @@ static ScmObj numread_error(const char *msg, struct numread_packet *context)
 
 ScmObj Scm_StringToNumber(ScmString *str, int radix, int strict)
 {
-    if (SCM_STRING_LENGTH(str) != SCM_STRING_SIZE(str)) {
+    u_int len, size;
+    const char *p = Scm_GetStringContent(str, &size, &len, NULL);
+    if (size != len) {
         /* This can't be a proper number. */
         return SCM_FALSE;
     } else {
-        return read_number(SCM_STRING_START(str), SCM_STRING_SIZE(str),
-                           radix, strict);
+        return read_number(p, size, radix, strict);
     }
 }
 
