@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: port.c,v 1.119 2005/05/24 23:28:38 shirok Exp $
+ *  $Id: port.c,v 1.122 2005/10/13 08:14:13 shirok Exp $
  */
 
 #include <unistd.h>
@@ -672,7 +672,7 @@ void Scm_FlushAllPorts(int exitting)
     ScmObj p = SCM_FALSE;
     int i, saved = 0;
 
-    save = SCM_WEAKVECTOR(Scm_MakeWeakVector(PORT_VECTOR_SIZE));
+    save = SCM_WEAK_VECTOR(Scm_MakeWeakVector(PORT_VECTOR_SIZE));
     ports = active_buffered_ports.ports;
     
     for (i=0; i<PORT_VECTOR_SIZE;) {
@@ -841,6 +841,12 @@ ScmObj Scm_OpenFilePort(const char *path, int flags, int buffering, int perm)
     if (buffering < SCM_PORT_BUFFER_FULL || buffering > SCM_PORT_BUFFER_NONE) {
         Scm_Error("bad buffering flag: %d", buffering);
     }
+#if defined(__MINGW32__)
+    /* Force binary mode if not specified */
+    if (!(flags & (O_TEXT|O_BINARY))) {
+	flags |= O_BINARY;
+    }
+#endif /*__MINGW32__*/
     fd = open(path, flags, perm);
     if (fd < 0) return SCM_FALSE;
     bufrec.mode = buffering;
@@ -893,9 +899,11 @@ ScmObj Scm_MakePortWithFd(ScmObj name, int direction,
 ScmObj Scm_MakeInputStringPort(ScmString *str, int privatep)
 {
     ScmPort *p = make_port(SCM_CLASS_PORT, SCM_PORT_INPUT, SCM_PORT_ISTR);
-    p->src.istr.start = SCM_STRING_START(str);
-    p->src.istr.current = SCM_STRING_START(str);
-    p->src.istr.end = SCM_STRING_START(str) + SCM_STRING_SIZE(str);
+    u_int size;
+    const char *s = Scm_GetStringContent(str, &size, NULL, NULL);
+    p->src.istr.start = s;
+    p->src.istr.current = s;
+    p->src.istr.end = s + size;
     SCM_PORT(p)->name = SCM_MAKE_STR("(input string port)");
     if (privatep) PORT_PRELOCK(p, Scm_VM());
     return SCM_OBJ(p);
@@ -918,7 +926,7 @@ ScmObj Scm_GetOutputString(ScmPort *port)
         Scm_Error("output string port required, but got %S", port);
     vm = Scm_VM();
     PORT_LOCK(port, vm);
-    r = Scm_DStringGet(&SCM_PORT(port)->src.ostr);
+    r = Scm_DStringGet(&SCM_PORT(port)->src.ostr, 0);
     PORT_UNLOCK(port);
     return r;
 }
@@ -927,7 +935,7 @@ ScmObj Scm_GetOutputStringUnsafe(ScmPort *port)
 {
     if (SCM_PORT_TYPE(port) != SCM_PORT_OSTR)
         Scm_Error("output string port required, but got %S", port);
-    return Scm_DStringGet(&SCM_PORT(port)->src.ostr);
+    return Scm_DStringGet(&SCM_PORT(port)->src.ostr, 0);
 }
 
 ScmObj Scm_GetRemainingInputString(ScmPort *port)
@@ -1355,7 +1363,7 @@ ScmObj Scm_Stderr(void)
 void Scm__InitPort(void)
 {
     (void)SCM_INTERNAL_MUTEX_INIT(active_buffered_ports.mutex);
-    active_buffered_ports.ports = SCM_WEAKVECTOR(Scm_MakeWeakVector(PORT_VECTOR_SIZE));
+    active_buffered_ports.ports = SCM_WEAK_VECTOR(Scm_MakeWeakVector(PORT_VECTOR_SIZE));
 
     Scm_InitStaticClass(&Scm_PortClass, "<port>",
                         Scm_GaucheModule(), NULL, 0);
