@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: regexp.c,v 1.52 2005/05/24 07:46:14 shirok Exp $
+ *  $Id: regexp.c,v 1.56 2005/11/02 10:49:19 shirok Exp $
  */
 
 #include <setjmp.h>
@@ -1253,7 +1253,7 @@ static void rc3_rec(regcomp_ctx *ctx, ScmObj ast, int lastp, int toplevelp)
         int count, n, j0 = 0, jn;
         int greedy = SCM_EQ(type, SCM_SYM_REP_BOUND);
 
-        SCM_ASSERT(Scm_Length(ast) == 3 && SCM_INTP(SCM_CADR(ast)));
+        SCM_ASSERT(Scm_Length(ast) >= 3 && SCM_INTP(SCM_CADR(ast)));
         count = SCM_INT_VALUE(SCM_CADR(ast));
         SCM_ASSERT(count > 0);
         item = SCM_CDDR(ast);
@@ -1527,6 +1527,7 @@ static ScmObj rc_setup_context(regcomp_ctx *ctx, ScmObj ast)
             return ast;
         }
         if (SCM_EQ(ast, SCM_SYM_BOL) || SCM_EQ(ast, SCM_SYM_EOL)
+            || SCM_EQ(ast, SCM_SYM_WB) || SCM_EQ(ast, SCM_SYM_NWB)
             || SCM_EQ(ast, SCM_SYM_ANY)) {
             return ast;
         }
@@ -1549,12 +1550,14 @@ static ScmObj rc_setup_context(regcomp_ctx *ctx, ScmObj ast)
     }
     if (SCM_EQ(type, SCM_SYM_SEQ) || SCM_EQ(type, SCM_SYM_ALT)
         || SCM_EQ(type, SCM_SYM_SEQ_UNCASE) || SCM_EQ(type, SCM_SYM_SEQ_CASE)
-        || SCM_EQ(type, SCM_SYM_REP) || SCM_EQ(type, SCM_SYM_REP_WHILE)) {
+        || SCM_EQ(type, SCM_SYM_REP) || SCM_EQ(type, SCM_SYM_REP_MIN)
+        || SCM_EQ(type, SCM_SYM_ASSERT) || SCM_EQ(type, SCM_SYM_NASSERT)
+        || SCM_EQ(type, SCM_SYM_REP_WHILE)) {
         rest = rc_setup_context_seq(ctx, SCM_CDR(ast));
         if (SCM_EQ(SCM_CDR(ast), rest)) return ast;
         else return Scm_Cons(type, rest);
     }
-    if (SCM_EQ(type, SCM_SYM_REP_BOUND)) {
+    if (SCM_EQ(type, SCM_SYM_REP_BOUND) || SCM_EQ(type, SCM_SYM_REP_BOUND_MIN)) {
         if (!SCM_PAIRP(SCM_CDR(ast)) || !SCM_INTP(SCM_CADR(ast))
             || SCM_INT_VALUE(SCM_CADR(ast)) < 0) {
             goto badast;
@@ -1599,10 +1602,9 @@ ScmObj Scm_RegComp(ScmString *pattern, int flags)
     if (SCM_STRING_INCOMPLETE_P(pattern)) {
         Scm_Error("incomplete string is not allowed: %S", pattern);
     }
-    rx->pattern = SCM_STRING(Scm_MakeString(SCM_STRING_START(pattern),
-                                            SCM_STRING_SIZE(pattern),
-                                            SCM_STRING_LENGTH(pattern),
-                                            SCM_MAKSTR_IMMUTABLE));
+    rx->pattern = SCM_STRING(Scm_CopyStringWithFlags(pattern, 
+                                                     SCM_STRING_IMMUTABLE,
+                                                     SCM_STRING_IMMUTABLE));
     rc_ctx_init(&cctx, rx);
     cctx.casefoldp = flags & SCM_REGEXP_CASE_FOLD;
     rx->flags |= (flags & SCM_REGEXP_CASE_FOLD);
@@ -1918,15 +1920,17 @@ static ScmObj make_match(ScmRegexp *rx, ScmString *orig,
     int i;
     struct match_list *ml;
     ScmRegMatch *rm = SCM_NEW(ScmRegMatch);
+    const ScmStringBody *origb;
     SCM_SET_CLASS(rm, SCM_CLASS_REGMATCH);
     rm->numMatches = rx->numGroups;
     rm->matches = SCM_NEW_ARRAY(struct ScmRegMatchSub, rx->numGroups);
     /* we keep information of original string separately, instead of
        keeping a pointer to orig; For orig may be destructively modified,
        but its elements are not. */
-    rm->input = SCM_STRING_START(orig);
-    rm->inputLen = SCM_STRING_LENGTH(orig);
-    rm->inputSize = SCM_STRING_SIZE(orig);
+    origb = SCM_STRING_BODY(orig);
+    rm->input = SCM_STRING_BODY_START(origb);
+    rm->inputLen = SCM_STRING_BODY_LENGTH(origb);
+    rm->inputSize = SCM_STRING_BODY_SIZE(origb);
     for (i=0; i<rx->numGroups; i++) {
         rm->matches[i].start = -1;
         rm->matches[i].length = -1;
@@ -1961,7 +1965,7 @@ static ScmObj rex(ScmRegexp *rx, ScmString *orig,
     sigjmp_buf cont;
     ctx.rx = rx;
     ctx.codehead = rx->code;
-    ctx.input = SCM_STRING_START(orig);
+    ctx.input = SCM_STRING_BODY_START(SCM_STRING_BODY(orig));
     ctx.stop = end;
     ctx.matches = NULL;
     ctx.begin_stack = (void*)&ctx;
@@ -1980,9 +1984,11 @@ static ScmObj rex(ScmRegexp *rx, ScmString *orig,
  */
 ScmObj Scm_RegExec(ScmRegexp *rx, ScmString *str)
 {
-    const char *start = SCM_STRING_START(str);
-    const char *end = start + SCM_STRING_SIZE(str);
-    int mustMatchLen = rx->mustMatch? SCM_STRING_SIZE(rx->mustMatch) : 0;
+    const ScmStringBody *b = SCM_STRING_BODY(str);
+    const char *start = SCM_STRING_BODY_START(b);
+    const char *end = start + SCM_STRING_BODY_SIZE(b);
+    const ScmStringBody *mb = rx->mustMatch? SCM_STRING_BODY(rx->mustMatch) : NULL;
+    int mustMatchLen = mb? SCM_STRING_BODY_SIZE(mb) : 0;
 
     if (SCM_STRING_INCOMPLETE_P(str)) {
         Scm_Error("incomplete string is not allowed: %S", str);
@@ -2035,7 +2041,7 @@ ScmObj Scm_RegMatchSubstr(ScmRegMatch *rm, int i)
                               sub->length, 0);
     } else {
         ScmObj s = Scm_MakeString(sub->startp, sub->endp - sub->startp, -1, 0);
-        sub->length = SCM_STRING_LENGTH(s);
+        sub->length = SCM_STRING_BODY_LENGTH(SCM_STRING_BODY(s));
         return s;
     }
 }
