@@ -1,7 +1,7 @@
 /*
  * signal.c - signal handling
  *
- *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: signal.c,v 1.31 2004/11/22 14:12:43 shirok Exp $
+ *  $Id: signal.c,v 1.33 2005/06/30 22:48:31 shirok Exp $
  */
 
 #include <stdlib.h>
@@ -92,9 +92,11 @@ static struct sigHandlersRec {
 /* Table of signals and its initial behavior. */
 #define SIGDEF_NOHANDLE 0       /* Gauche doesn't install a signal handler,
                                    leaving it to the application. */
-#define SIGDEF_ERROR    1       /* Gauche installs a default signal handler
+#define SIGDEF_DFL      1       /* Gauche resets the singal handler to
+                                   SIG_DFL. */
+#define SIGDEF_ERROR    2       /* Gauche installs a default signal handler
                                    that raises an error. */
-#define SIGDEF_EXIT     2       /* Gauche installs a handler that calls
+#define SIGDEF_EXIT     3       /* Gauche installs a handler that calls
                                    Scm_Exit(). */
 
 #define SIGDEF(x, flag)  { #x, x, flag }
@@ -144,7 +146,7 @@ static struct sigdesc {
     SIGDEF(SIGSTKFLT, SIGDEF_ERROR),  /* Stack fault */
 #endif
 #ifdef SIGCHLD
-    SIGDEF(SIGCHLD, SIGDEF_NOHANDLE), /* Child status has changed (POSIX) */
+    SIGDEF(SIGCHLD, SIGDEF_DFL),      /* Child status has changed (POSIX) */
 #endif
 #ifdef SIGCONT
     SIGDEF(SIGCONT, SIGDEF_NOHANDLE), /* Continue (POSIX) */
@@ -267,7 +269,7 @@ static ScmObj default_sighandler(ScmObj *args, int nargs, void *data)
 }
 
 static SCM_DEFINE_STRING_CONST(default_sighandler_name,
-                               "%default-signal-handler", 24, 24);
+                               "%default-signal-handler", 23, 23);
 static SCM_DEFINE_SUBR(default_sighandler_stub, 1, 0,
                        SCM_OBJ(&default_sighandler_name),
                        default_sighandler,
@@ -291,6 +293,22 @@ static SCM_DEFINE_SUBR(exit_sighandler_stub, 1, 0,
 
 #define EXIT_SIGHANDLER    SCM_OBJ(&exit_sighandler_stub)
 
+/* For some signals, gauche does nothing */
+static ScmObj through_sighandler(ScmObj *args, int nargs, void *data)
+{
+    return SCM_UNDEFINED;
+}
+
+static SCM_DEFINE_STRING_CONST(through_sighandler_name,
+                               "%through-signal-handler", 20, 20);
+static SCM_DEFINE_SUBR(through_sighandler_stub, 1, 0,
+                       SCM_OBJ(&through_sighandler_name),
+                       through_sighandler,
+                       NULL, NULL);
+
+#define THROUGH_SIGHANDLER    SCM_OBJ(&through_sighandler_stub)
+
+
 /*
  * sigset class
  */
@@ -583,14 +601,24 @@ void Scm_SetMasterSigmask(sigset_t *set)
         } else if (!sigismember(&sigHandlers.masterSigset, desc->num)
                    && sigismember(set, desc->num)) {
             /* add sighandler if necessary */
-            if (desc->defaultHandle != SIGDEF_NOHANDLE) {
+            if (desc->defaultHandle == SIGDEF_DFL) {
+                if (sigaction(desc->num, &actoff, NULL) != 0) {
+                    Scm_SysError("sigaction on %d failed", desc->num);
+                }
+                sigHandlers.handlers[desc->num] = SCM_TRUE;
+            } else if (desc->defaultHandle != SIGDEF_NOHANDLE) {
                 if (sigaction(desc->num, &acton, NULL) != 0) {
                     Scm_SysError("sigaction on %d failed", desc->num);
                 }
-                if (desc->defaultHandle == SIGDEF_EXIT) {
-                    sigHandlers.handlers[desc->num] = EXIT_SIGHANDLER;
-                } else {
+                switch (desc->defaultHandle) {
+                case SIGDEF_ERROR:
                     sigHandlers.handlers[desc->num] = DEFAULT_SIGHANDLER;
+                    break;
+                case SIGDEF_EXIT:
+                    sigHandlers.handlers[desc->num] = EXIT_SIGHANDLER;
+                    break;
+                default:
+                    Scm_Panic("Scm_SetMasterSigmask: can't be here");
                 }
             }
         }
