@@ -1,7 +1,7 @@
 /*
  * symbol.c - symbol implementation
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: symbol.c,v 1.32 2005/04/27 02:06:44 shirok Exp $
+ *  $Id: symbol.c,v 1.37 2005/10/13 08:14:13 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -59,7 +59,8 @@ ScmObj Scm_Intern(ScmString *name)
     ScmHashEntry *e = Scm_HashTableGet(obtable, SCM_OBJ(name));
     if (e) return e->value;
     else {
-        ScmObj n = Scm_StringMakeImmutable(SCM_STRING(Scm_CopyString(name)));
+        ScmObj n = Scm_CopyStringWithFlags(name, SCM_STRING_IMMUTABLE,
+                                           SCM_STRING_IMMUTABLE);
         ScmSymbol *sym;
         INITSYM(sym, n);
         Scm_HashTablePut(obtable, n, SCM_OBJ(sym));
@@ -123,8 +124,9 @@ static void symbol_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
         /* TODO: For now, we regard chars over 0x80 is all "printable".
            Need a more consistent mechanism. */
         ScmString *snam = SCM_SYMBOL(obj)->name;
-        const char *p = SCM_STRING_START(snam), *q;
-        int siz = SCM_STRING_SIZE(snam), i;
+        const ScmStringBody *b = SCM_STRING_BODY(snam);
+        const char *p = SCM_STRING_BODY_START(b), *q;
+        int siz = SCM_STRING_BODY_SIZE(b), i;
         int escape = FALSE;
         int case_mask =
             ((SCM_WRITE_CASE(ctx) == SCM_WRITE_CASE_FOLD)? 0x12 : 0x02);
@@ -182,7 +184,9 @@ static void symbol_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 static void gloc_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     ScmGloc *g = SCM_GLOC(obj);
-    Scm_Printf(port, "#<gloc %S#%S>", g->module->name, g->name);
+    Scm_Printf(port, "#<gloc %S%s%S>", g->module->name,
+               (g->exported?"#":"##"),
+               g->name);
 }
 
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_GlocClass, gloc_print);
@@ -194,6 +198,7 @@ ScmObj Scm_MakeGloc(ScmSymbol *sym, ScmModule *module)
     g->name = sym;
     g->module = module;
     g->value = SCM_UNBOUND;
+    g->exported = FALSE;
     g->getter = NULL;
     g->setter = NULL;
     return SCM_OBJ(g);
@@ -221,6 +226,6 @@ ScmObj Scm_GlocConstSetter(ScmGloc *gloc, ScmObj val)
 
 void Scm__InitSymbol(void)
 {
-    obtable = SCM_HASHTABLE(Scm_MakeHashTable((ScmHashProc)SCM_HASH_STRING, NULL, 2000));
+    obtable = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_STRING, 4096));
     init_builtin_syms();
 }
