@@ -1,7 +1,7 @@
 /*
  * system.c - system interface
  *
- *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: system.c,v 1.63 2005/05/24 23:28:38 shirok Exp $
+ *  $Id: system.c,v 1.71 2005/10/28 02:53:10 shirok Exp $
  */
 
 #include <stdio.h>
@@ -186,14 +186,14 @@ ScmObj Scm_ReadDirectory(ScmString *pathname)
 /* TODO: allow to take optional flags */
 ScmObj Scm_GlobDirectory(ScmString *pattern)
 {
-#ifdef HAVE_GLOB_H
+#if defined(HAVE_GLOB_H)
     glob_t globbed;
     ScmObj head = SCM_NIL, tail = SCM_NIL;
     int i, r;
     SCM_SYSCALL(r, glob(Scm_GetStringConst(pattern), 0, NULL, &globbed));
     if (r) {
         globfree(&globbed);
-#ifdef GLOB_NOMATCH
+#if defined(GLOB_NOMATCH)
         if (r == GLOB_NOMATCH) return SCM_NIL;
 #endif /*!GLOB_NOMATCH*/
         Scm_Error("Couldn't glob %S", pattern);
@@ -204,14 +204,44 @@ ScmObj Scm_GlobDirectory(ScmString *pattern)
     }
     globfree(&globbed);
     return head;
-#else  /*!HAVE_GLOB_H*/
+#elif defined(__MINGW32__)
+    /* We provide alternative using Windows API */
+    HANDLE dirp;
+    WIN32_FIND_DATA fdata;
+    DWORD winerrno;
+    const char *path = Scm_GetStringConst(pattern);
+    ScmObj head = SCM_NIL, tail = SCM_NIL;
+    
+    dirp = FindFirstFile(path, &fdata);
+    if (dirp == INVALID_HANDLE_VALUE) {
+	if ((winerrno = GetLastError()) != ERROR_FILE_NOT_FOUND) goto err;
+	return head;
+    }
+    SCM_APPEND1(head, tail, SCM_MAKE_STR_COPYING(fdata.cFileName));
+    while (FindNextFile(dirp, &fdata) != 0) {
+	SCM_APPEND1(head, tail, SCM_MAKE_STR_COPYING(fdata.cFileName));
+    }
+    winerrno = GetLastError();
+    FindClose(dirp);
+    if (winerrno != ERROR_NO_MORE_FILES) goto err;
+    return head;
+ err:
+    Scm_Error("Searching directory failed by windows error %d",
+	      winerrno);
+    return SCM_UNDEFINED;	/* dummy */
+#else  /*!HAVE_GLOB_H && !__MINGW32__*/
     Scm_Error("glob-directory is not supported on this architecture.");
     return SCM_UNDEFINED;
-#endif /*!HAVE_GLOB_H*/
+#endif /*!HAVE_GLOB_H && !__MINGW32__*/
 }
 
-/*
+/*===============================================================
  * Pathname manipulation
+ *
+ *  It gets complicated since the byte '/' and '\\' can appear in
+ *  the trailing octets of a multibyte character.
+ *  Assuming these operations won't be a bottleneck, we use simple and
+ *  straightforward code rather than tricky and fast one.
  */
 
 /* Returns the system's native pathname delimiter. */
@@ -224,157 +254,299 @@ const char *Scm_PathDelimiter(void)
 #endif /*__MINGW32__*/
 }
 
-ScmObj Scm_NormalizePathname(ScmString *pathname, int flags)
-{
-    const char *str = SCM_STRING_START(pathname), *srcp = str;
-    int size = SCM_STRING_SIZE(pathname);
-    char *buf = NULL, *dstp;
-    int bottomp = FALSE;
-    
-#define SKIP_SLASH \
-    while ((*srcp == '/' || *srcp == '\\') && srcp < str+size) { srcp++; }
+#ifdef __MINGW32__
+#define SEPARATOR '\\'
+#define ROOTDIR   "\\"
+#else
+#define SEPARATOR '/'
+#define ROOTDIR   "/"
+#endif
 
-    if ((flags & SCM_PATH_EXPAND) && size >= 1 && *str == '~') {
-#ifndef __MINGW32__
-        /* ~user magic */
-        const char *p = str+1;
-        struct passwd *pwd;
-        int dirlen;
-        
-        for (; p < str+size && *p != '/' && *p != '\0'; p++)
-            ;
-        if (p == str+1) {
-            pwd = getpwuid(geteuid());
-            if (pwd == NULL) {
-                Scm_SigCheck(Scm_VM());
-                Scm_SysError("couldn't get home directory.\n");
-            }
-        } else {
-            char *user = (char *)SCM_MALLOC_ATOMIC(p-str);
-            memcpy(user, str+1, p-str-1);
-            user[p-str-1] = '\0';
-            pwd = getpwnam(user);
-            if (pwd == NULL) {
-                Scm_SigCheck(Scm_VM());
-                Scm_Error("couldn't get home directory of user \"%s\".\n",
-                          user);
-            }
-        }
-        srcp = p;
-        SKIP_SLASH;
-        dirlen = strlen(pwd->pw_dir);
-        buf = SCM_NEW_ATOMIC2(char*, dirlen+size+1);
-        strcpy(buf, pwd->pw_dir);
-        dstp = buf + dirlen;
-        if (*(dstp-1) != '/') { *dstp++ = '/'; *(dstp+1) = '\0'; }
-#else  /* __MINGW32__ */
-	/* we don't support '~' magic on win32 native */
-	buf = SCM_NEW_ATOMIC2(char*, size+1);
-	dstp = buf;
-#endif /* __MINGW32__ */
-    } else if ((flags & SCM_PATH_ABSOLUTE) && *str != '/') {
-        int dirlen;
-#define GETCWD_PATH_MAX 1024  /* TODO: must be configured */
-        char p[GETCWD_PATH_MAX];
-        if (getcwd(p, GETCWD_PATH_MAX-1) == NULL) {
+/* Returns the pointer to the first path separator character,
+   or NULL if no separator is found. */
+static const char *get_first_separator(const char *path, const char *end)
+{
+    const char *p = path;
+    while (p < end) {
+        if (*p == '/' || *p == '\\') return p;
+        p += SCM_CHAR_NFOLLOWS(*p)+1;
+    }
+    return NULL;
+}
+
+/* Returns the pointer to the last path separator character,
+   or NULL if no separator is found. */
+static const char *get_last_separator(const char *path, const char *end)
+{
+    const char *p = path, *last = NULL;
+    while (p < end) {
+        if (*p == '/' || *p == '\\') last = p;
+        p += SCM_CHAR_NFOLLOWS(*p)+1;
+    }
+    return last;
+}
+
+static const char *skip_separators(const char *p, const char *end)
+{
+    while (p < end) {
+        if (*p != '/' && *p != '\\') break;
+        p += SCM_CHAR_NFOLLOWS(*p)+1;
+    }
+    return p;
+}
+
+/* Returns the end pointer sans trailing separators. */
+static const char *truncate_trailing_separators(const char *path,
+                                                const char *end)
+{
+    const char *p = get_first_separator(path, end), *q;
+    if (p == NULL) return end;
+    for (;;) {
+        q = skip_separators(p, end);
+        if (q == end) return p;
+        p = get_first_separator(q, end);
+        if (p == NULL) return end;
+    }
+}
+
+/* A utility for tilde expansion.  They are called only on
+   Unix variants, so we only need to check '/'. */
+static void put_user_home(ScmDString *dst,
+                          const char *name,
+                          const char *end)
+{
+    struct passwd *pwd;
+    int dirlen;
+
+    if (name == end) {
+        pwd = getpwuid(geteuid());
+        if (pwd == NULL) {
             Scm_SigCheck(Scm_VM());
-            Scm_SysError("couldn't get current directory.");
-        }
-        dirlen = strlen(p);
-        buf = SCM_NEW_ATOMIC2(char*, dirlen+size+1);
-        strcpy(buf, p);
-        dstp = buf + dirlen;
-        if (*(dstp-1) != '/') *dstp++ = '/';
-    } else if (flags & SCM_PATH_CANONICALIZE) {
-        dstp = buf = SCM_NEW_ATOMIC2(char*, size+1);
-        if (*str == '/') {
-            *dstp++ = '/';
-            SKIP_SLASH;
+            Scm_SysError("couldn't get home directory.\n");
         }
     } else {
-        return SCM_OBJ(pathname); /* nothing to do */
+        int namesiz = end - name;
+        char *uname = (char*)SCM_MALLOC_ATOMIC(namesiz+1);
+        memcpy(uname, name, namesiz);
+        uname[namesiz] = '\0';
+        pwd = getpwnam(uname);
+        if (pwd == NULL) {
+            Scm_SigCheck(Scm_VM());
+            Scm_Error("couldn't get home directory of user \"%s\".\n", uname);
+        }
     }
+    dirlen = strlen(pwd->pw_dir);
+    Scm_DStringPutz(dst, pwd->pw_dir, dirlen);
+    if (pwd->pw_dir[dirlen-1] != '/') Scm_DStringPutc(dst, '/');
+}
 
+/* SRC points to the pathname string beginning with '~'.  Expand it
+   to the user's home directory, leaving the partial result in DST.
+   Returns the pointer into SRC sans tilde prefix (e.g. removed "~user/").
+   The returned pointer may points just past the last char of SRC. */
+static const char *expand_tilde(ScmDString *dst,
+                                const char *src,
+                                const char *end)
+{
+    struct passwd *pwd;
+    int dirlen;
+    const char *sep = get_first_separator(src, end);
+
+    if (sep == NULL) {
+        put_user_home(dst, src+1, end);
+        return end;
+    } else {
+        put_user_home(dst, src+1, sep);
+        return skip_separators(sep, end);
+    }
+}
+
+/* Put current dir to DST */
+static void put_current_dir(ScmDString *dst)
+{
+    int dirlen;
+#define GETCWD_PATH_MAX 1024  /* TODO: must be configured */
+    char p[GETCWD_PATH_MAX];
+    if (getcwd(p, GETCWD_PATH_MAX-1) == NULL) {
+        Scm_SigCheck(Scm_VM());
+        Scm_SysError("couldn't get current directory.");
+    }
+    dirlen = strlen(p);
+    Scm_DStringPutz(dst, p, dirlen);
+    if (p[dirlen-1] != '/' && p[dirlen-1] != '\\') {
+        Scm_DStringPutc(dst, SEPARATOR);
+    }
+#undef GETCWD_PATH_MAX
+}
+
+/* win32 specific; copy pathname with replacing '/' by '\\'. */
+static void copy_win32_path(ScmDString *dst,
+                            const char *srcp,
+                            const char *end)
+{
+    while (srcp < end) {
+        ScmChar ch;
+        if (*srcp == '/' || *srcp == '\\') {
+            Scm_DStringPutc(dst, SEPARATOR);
+        } else {
+            SCM_CHAR_GET(srcp, ch);
+            Scm_DStringPutc(dst, ch);
+        }
+        srcp += SCM_CHAR_NBYTES(ch);
+    }
+}
+
+ScmObj Scm_NormalizePathname(ScmString *pathname, int flags)
+{
+    u_int size;
+    const char *str = Scm_GetStringContent(pathname, &size, NULL, NULL);
+    const char *srcp = str;
+    const char *endp = str + size;
+    ScmDString buf;
+
+    Scm_DStringInit(&buf);
+
+    /* Preprocess.  We expand tilde (on unix platform), and prepend the
+       current directory to the relative pathname if absolutize is required.
+       For canonicalization, we also put any absolute prefix into buf, so
+       that srcp points to the relative path part after this. */
+#if !defined(__MINGW32__)
+    if ((flags & SCM_PATH_EXPAND) && size >= 1 && *str == '~') {
+        srcp = expand_tilde(&buf, srcp, endp);
+    } else if (endp > srcp && *srcp == '/') {
+        /* Path is absolute */
+        if (flags & SCM_PATH_CANONICALIZE) {
+            Scm_DStringPutc(&buf, SEPARATOR);
+            srcp = skip_separators(srcp, endp);
+        }
+    } else {
+        /* Path is relative */
+        if (flags & SCM_PATH_ABSOLUTE) {
+            put_current_dir(&buf);
+        }
+    }
     if (!(flags & SCM_PATH_CANONICALIZE)) {
-        size -= srcp-str;
-        memcpy(dstp, srcp, size);
-        *(dstp + size) = '\0';
-        return Scm_MakeString(buf, (dstp-buf)+size, -1, SCM_MAKSTR_COPYING);
+        Scm_DStringPutz(&buf, srcp, endp - srcp);
+	return Scm_DStringGet(&buf, 0);
     }
+#else /* __MINGW32__ */
+    if (endp > srcp+1 && isalpha(*srcp) && *(srcp+1) == ':') {
+        /* We first process the Evil Drive Letter */
+        Scm_DStringPutc(&buf, *srcp++);
+        Scm_DStringPutc(&buf, *srcp++);
+    }
+    if (endp > srcp && (*srcp == '/' || *srcp == '\\')) {
+        if (flags & SCM_PATH_CANONICALIZE) {
+            Scm_DStringPutc(&buf, SEPARATOR);
+            srcp = skip_separators(srcp, endp);
+        }
+    } else if (srcp == str) {
+        /* Path is relative (the srcp==str condition rules out the
+           drive letter case */
+        if (flags & SCM_PATH_ABSOLUTE) {
+            put_current_dir(&buf);
+        }
+    }
+    if (!(flags & SCM_PATH_CANONICALIZE)) {
+        copy_win32_path(&buf, srcp, endp);
+	return Scm_DStringGet(&buf, 0);
+    }
+#endif /* __MINGW32__ */
 
-    while (srcp < str+size) {
-        if (*srcp == '.') {
-            if (srcp == str+size-1) {
-                *dstp++ = '.'; /* preserve the last dot */
+    /* Canonicalization.  We used to have a tricky piece of code here
+       that avoids extra allocations, but have replaced it for
+       simple-minded code, since speed gain here won't contribute to
+       the overall performance anyway.  */
+    {
+        ScmObj comps = SCM_NIL; /* reverse list of components */
+        int cnt = 0;            /* # of components except ".."'s */
+        int final = FALSE;
+        int wentup = FALSE;     /* true if the last loop went up a dir */
+        const char *p;
+
+        for (;;) {
+            p = get_first_separator(srcp, endp);
+            if (p == NULL) {
+                final = TRUE;
+                p = endp;
+            }
+
+            if (p == srcp+1 && *srcp == '.') {
+                /* do nothing */
+            } else if (p == srcp+2 && srcp[0] == '.' && srcp[1] == '.') {
+                if (cnt > 0) {
+                    SCM_ASSERT(SCM_PAIRP(comps));
+                    comps = SCM_CDR(comps);
+                    cnt--;
+                    wentup = TRUE;
+                } else {
+                    comps = Scm_Cons(SCM_MAKE_STR(".."), comps);
+                    wentup = FALSE;
+                }
+            } else {
+                comps = Scm_Cons(Scm_MakeString(srcp, p-srcp, -1, 0), comps);
+                cnt++;
+                wentup = FALSE;
+            }
+            if (final) {
+                /* If we just went up a directory, we want to preserve the
+                   trailing separator in the result.  So we add an empty
+                   component. */
+                if (wentup) comps = Scm_Cons(SCM_MAKE_STR(""), comps);
                 break;
             }
-            if (*(srcp+1) == '/') {
-                srcp++;
-                SKIP_SLASH;
-                continue;
-            }
-            if (!bottomp
-                && *(srcp+1) == '.'
-                && (srcp == str+size-2 || *(srcp+2) == '/')) {
-
-                /* back up to parent dir */
-                char *q = dstp-2;
-                for (;q >= buf; q--) {
-                    if (*q == '/') break;
-                }
-                if (q >= buf) {
-                    dstp = q+1;
-                } else {
-                    bottomp = TRUE;
-                    *dstp++ = '.';
-                    *dstp++ = '.';
-                    *dstp++ = '/';
-                }
-                srcp += 3;
-                continue;
+            srcp = skip_separators(p, endp);
+        }
+        if (SCM_PAIRP(comps)) {
+            comps = Scm_ReverseX(comps);
+            Scm_DStringAdd(&buf, SCM_STRING(SCM_CAR(comps)));
+            SCM_FOR_EACH(comps, SCM_CDR(comps)) {
+                Scm_DStringPutc(&buf, SEPARATOR);
+                Scm_DStringAdd(&buf, SCM_STRING(SCM_CAR(comps)));
             }
         }
-        while ((*dstp++ = *srcp++) != '/' && srcp < str+size)
-            ;
-        SKIP_SLASH;
     }
-    *dstp = '\0';
-    return Scm_MakeString(buf, dstp-buf, -1, SCM_MAKSTR_COPYING);
+    return Scm_DStringGet(&buf, 0);        
 }
 
 ScmObj Scm_BaseName(ScmString *filename)
 {
-    const char *p, *str = SCM_STRING_START(filename);
-    int i, size = SCM_STRING_SIZE(filename);
+    u_int size;
+    const char *path = Scm_GetStringContent(filename, &size, NULL, NULL);
+    const char *endp, *last;
 
     if (size == 0) return SCM_MAKE_STR("");
-    p = str+size-1;
-    while (*p == '/' && size > 0) { p--; size--; } /* ignore trailing '/' */
-    if (size == 0) return SCM_MAKE_STR("");
-    for (i = 0; i < size; i++, p--) {
-        if (*p == '/') break;
+    endp = truncate_trailing_separators(path, path+size);
+    last = get_last_separator(path, endp);
+    if (last == NULL) {
+        return Scm_MakeString(path, endp-path, -1, 0);
+    } else {
+        return Scm_MakeString(last+1, endp-last-1, -1, 0);
     }
-    return Scm_MakeString(p+1, i, -1, 0);
 }
 
 ScmObj Scm_DirName(ScmString *filename)
 {
-    const char *p, *str = SCM_STRING_START(filename);
-    int i, size = SCM_STRING_SIZE(filename);
+    u_int size;
+    const char *path = Scm_GetStringContent(filename, &size, NULL, NULL);
+    const char *endp, *last;
 
     if (size == 0) return SCM_MAKE_STR(".");
-    p = str+size-1;
-    while (*p == '/' && size > 0) { p--; size--; } /* ignore trailing '/' */
-    if (size == 0) return SCM_MAKE_STR("/");
-    for (i = size; i > 0; i--, p--) {
-        if (*p == '/') break;
+    endp = truncate_trailing_separators(path, path+size);
+    if (endp == path) return SCM_MAKE_STR(ROOTDIR);
+    last = get_last_separator(path, endp);
+    if (last == NULL) {
+        return SCM_MAKE_STR(".");
     }
-    if (i == 0) return SCM_MAKE_STR(".");
-    while (*p == '/' && i > 0) { p--; i--; } /* delete trailing '/' */
-    if (i == 0) return SCM_MAKE_STR("/");
-    return Scm_MakeString(str, i, -1, 0);
+
+    /* we have "something/", and 'last' points to the last separator. */
+    last = truncate_trailing_separators(path, last);
+    return Scm_MakeString(path, last-path, -1, 0);
 }
 
+#undef ROOTDIR
+#undef SEPARATOR
+
 /* Make mkstemp() work even if the system doesn't have one. */
 int Scm_Mkstemp(char *templat)
 {
@@ -392,16 +564,17 @@ int Scm_Mkstemp(char *templat)
 #define MKSTEMP_MAX_TRIALS 65535   /* avoid infinite loop */
     {
 	u_long seed = (u_long)time(NULL);
-	int numtry;
+	int numtry, flags;
 	char suffix[7];
+#if defined(__MINGW32__)
+	flags = O_CREAT|O_EXCL|O_WRONLY|O_BINARY;
+#else  /* !__MINGW32__ */
+	flags = O_CREAT|O_EXCL|O_WRONLY;
+#endif /* !__MINGW32__ */
 	for (numtry=0; numtry<MKSTEMP_MAX_TRIALS; numtry++) {
 	    snprintf(suffix, 7, "%06x", seed&0xffffff);
 	    memcpy(templat+siz-6, suffix, 7);
-#ifndef __MINGW32__
-	    SCM_SYSCALL(fd, open(templat, O_CREAT|O_EXCL|O_WRONLY, 0600));
-#else  /*__MINGW32__*/
- 	    SCM_SYSCALL(fd, open(templat, _O_CREAT|_O_EXCL|_O_WRONLY, 0600));
-#endif /*__MINGW32__*/
+	    SCM_SYSCALL(fd, open(templat, flags, 0600));
 	    if (fd >= 0) break;
 	    seed *= 2654435761UL;
 	}
@@ -419,11 +592,13 @@ ScmObj Scm_SysMkstemp(ScmString *templat)
 #define MKSTEMP_PATH_MAX 1025  /* Geez, remove me */
     char name[MKSTEMP_PATH_MAX];
     ScmObj sname;
-    int siz = SCM_STRING_SIZE(templat), fd;
+    u_int siz;
+    int fd;
+    const char *t = Scm_GetStringContent(templat, &siz, NULL, NULL);
     if (siz >= MKSTEMP_PATH_MAX-6) { 
 	Scm_Error("pathname too long: %S", templat);
     }
-    memcpy(name, SCM_STRING_START(templat), siz);
+    memcpy(name, t, siz);
     memcpy(name + siz, "XXXXXX", 6);
     name[siz+6] = '\0';
     fd = Scm_Mkstemp(name);
@@ -537,8 +712,35 @@ static void time_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     Scm_Printf(port, "#<%S %lu.%09lu>", t->type, t->sec, t->nsec);
 }
 
+static int time_compare(ScmObj x, ScmObj y, int equalp)
+{
+    ScmTime *tx = SCM_TIME(x);
+    ScmTime *ty = SCM_TIME(y);
+    
+    if (equalp) {
+        if (SCM_EQ(tx->type, ty->type)
+            && tx->sec == ty->sec
+            && tx->nsec == ty->nsec) {
+            return 0;
+        } else {
+            return 1;
+        }
+    } else {
+        if (!SCM_EQ(tx->type, ty->type)) {
+            Scm_Error("cannot compare different types of time objects: %S vs %S", x, y);
+        }
+        if (tx->sec < ty->sec) return -1;
+        if (tx->sec == ty->sec) {
+            if (tx->nsec < ty->nsec) return -1;
+            if (tx->nsec == ty->nsec) return 0;
+            else return 1;
+        } 
+        else return 1;
+    }
+}
+
 SCM_DEFINE_BUILTIN_CLASS(Scm_TimeClass,
-                         time_print, NULL, NULL,
+                         time_print, time_compare, NULL,
                          time_allocate, SCM_CLASS_DEFAULT_CPL);
 
 ScmObj Scm_MakeTime(ScmObj type, long sec, long nsec)
@@ -616,7 +818,7 @@ static void time_nsec_set(ScmTime *t, ScmObj val)
         Scm_Error("real number required, but got %S", val);
     }
     if ((l = Scm_GetInteger(val)) >= 1000000000) {
-        Scm_Error("nanoseconds out of range: %l", l);
+        Scm_Error("nanoseconds out of range: %ld", l);
     }
     t->nsec = l;
 }
