@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: write.c,v 1.52 2005/05/24 23:28:38 shirok Exp $
+ *  $Id: write.c,v 1.57 2005/10/13 08:14:13 shirok Exp $
  */
 
 #include <stdio.h>
@@ -142,13 +142,13 @@ void Scm_Write(ScmObj obj, ScmObj p, int mode)
 
     /* if this is a "walk" pass of write/ss, dispatch to the walker */
     if (port->flags & SCM_PORT_WALKING) {
-        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASHTABLEP(SCM_CDR(port->data)));
+        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASH_TABLE_P(SCM_CDR(port->data)));
         write_walk(obj, port, &ctx);
         return;
     }
     /* if this is a "output" pass of write/ss, call the recursive routine */
     if (port->flags & SCM_PORT_WRITESS) {
-        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASHTABLEP(SCM_CDR(port->data)));
+        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASH_TABLE_P(SCM_CDR(port->data)));
         write_ss_rec(obj, port, &ctx);
         return;
     }
@@ -224,7 +224,7 @@ int Scm_WriteCircular(ScmObj obj, ScmObj port, int mode, int width)
         ctx.limit = width;
     }
     ctx.ncirc = 0;
-    ctx.table = SCM_HASHTABLE(Scm_MakeHashTable(SCM_HASH_ADDRESS, NULL, 8));
+    ctx.table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 8));
 
     if (width > 0) {
         ScmObj out = Scm_MakeOutputStringPort(TRUE);
@@ -329,7 +329,7 @@ static ScmPort *make_walker_port(void)
                                           
     port = SCM_PORT(Scm_MakeVirtualPort(SCM_CLASS_PORT, SCM_PORT_OUTPUT,
                                         &walker_port_vtable));
-    ht = Scm_MakeHashTable(SCM_HASH_ADDRESS, NULL, 0);
+    ht = Scm_MakeHashTableSimple(SCM_HASH_EQ, 0);
     port->data = Scm_Cons(SCM_MAKE_INT(0), ht);
     port->flags = SCM_PORT_WALKING;
     return port;
@@ -342,7 +342,7 @@ static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     ScmHashTable *ht;
     ScmObj elt;
     
-    ht = SCM_HASHTABLE(SCM_CDR(port->data));
+    ht = SCM_HASH_TABLE(SCM_CDR(port->data));
 
     for (;;) {
         if (!SCM_PTRP(obj) || SCM_SYMBOLP(obj) || SCM_KEYWORDP(obj)
@@ -360,7 +360,7 @@ static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
             obj = SCM_CDR(obj);
             continue;
         }
-        if (SCM_STRINGP(obj) && SCM_STRING_SIZE(obj) > 0) {
+        if (SCM_STRINGP(obj) && !SCM_STRING_NULL_P(obj)) {
             e = Scm_HashTableGet(ht, obj);
             if (e) { e->value = SCM_TRUE; return; }
             Scm_HashTablePut(ht, obj, SCM_FALSE);
@@ -403,8 +403,8 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
         if (outlen(port) >= ctx->limit) return;
     }
 
-    if (SCM_PAIRP(port->data) && SCM_HASHTABLEP(SCM_CDR(port->data))) {
-        ht = SCM_HASHTABLE(SCM_CDR(port->data));
+    if (SCM_PAIRP(port->data) && SCM_HASH_TABLE_P(SCM_CDR(port->data))) {
+        ht = SCM_HASH_TABLE(SCM_CDR(port->data));
     }
 
     if (!SCM_PTRP(obj)) {
@@ -445,7 +445,7 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
         return;
     }
     
-    if ((SCM_STRINGP(obj) && SCM_STRING_SIZE(obj) == 0)
+    if ((SCM_STRINGP(obj) && SCM_STRING_NULL_P(obj))
         || (SCM_VECTORP(obj) && SCM_VECTOR_SIZE(obj) == 0)) {
         /* special case where we don't put a reference tag. */
         write_general(obj, port, ctx);
@@ -579,12 +579,12 @@ static void format_write(ScmObj obj, ScmPort *port, ScmWriteContext *ctx,
                          int sharedp)
 {
     if (port->flags & SCM_PORT_WALKING) {
-        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASHTABLEP(SCM_CDR(port->data)));
+        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASH_TABLE_P(SCM_CDR(port->data)));
         write_walk(obj, port, ctx);
         return;
     }
     if (port->flags & SCM_PORT_WRITESS) {
-        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASHTABLEP(SCM_CDR(port->data)));
+        SCM_ASSERT(SCM_PAIRP(port->data)&&SCM_HASH_TABLE_P(SCM_CDR(port->data)));
         write_ss_rec(obj, port, ctx);
         return;
     }
@@ -600,7 +600,7 @@ static void format_pad(ScmPort *out, ScmString *str,
                        int mincol, int colinc, ScmChar padchar,
                        int rightalign)
 {
-    int padcount = mincol- SCM_STRING_LENGTH(str);
+    int padcount = mincol- SCM_STRING_BODY_LENGTH(SCM_STRING_BODY(str));
     int i;
     
     if (padcount > 0) {
@@ -649,7 +649,7 @@ static void format_sexp(ScmPort *out, ScmObj arg,
     tmpstr = SCM_STRING(Scm_GetOutputString(SCM_PORT(tmpout)));
 
     if (maxcol > 0 && nwritten < 0) {
-        const char *s = SCM_STRING_START(tmpstr), *e;
+        const char *s = Scm_GetStringContent(tmpstr, NULL, NULL, NULL), *e;
         if (dots && maxcol > 4) {
             e = Scm_StringPosition(tmpstr, maxcol-4);
             Scm_PutzUnsafe(s, e-s, out);
@@ -685,7 +685,7 @@ static void format_integer(ScmPort *out, ScmObj arg,
     if (nparams>2 && SCM_CHARP(params[2])) commachar = SCM_CHAR_VALUE(params[2]);
     if (nparams>3 && SCM_INTP(params[3])) commainterval = SCM_INT_VALUE(params[3]);
     str = Scm_NumberToString(arg, radix, use_upper);
-    if (alwayssign && SCM_STRING_START(str)[0] != '-') {
+    if (alwayssign && SCM_STRING_BODY_START(SCM_STRING_BODY(str))[0] != '-') {
         str = Scm_StringAppend2(SCM_STRING(SCM_MAKE_STR("+")),
                                 SCM_STRING(str));
     }
@@ -696,8 +696,9 @@ static void format_integer(ScmPort *out, ScmObj arg,
            contradicts its examples; it is ambiguous about what happens
            if the number is padded. */
         ScmDString tmpout;
-        const char *ptr = SCM_STRING_START(str);
-        int num_digits = SCM_STRING_LENGTH(str), colcnt;
+        u_int num_digits, colcnt;
+        const char *ptr = Scm_GetStringContent(SCM_STRING(str), &num_digits,
+                                               NULL, NULL);
 
         Scm_DStringInit(&tmpout);
         if (*ptr == '-' || *ptr == '+') {
@@ -712,7 +713,7 @@ static void format_integer(ScmPort *out, ScmObj arg,
             Scm_DStringPutz(&tmpout, ptr+colcnt, commainterval);
             colcnt += commainterval;
         }
-        str = Scm_DStringGet(&tmpout);
+        str = Scm_DStringGet(&tmpout, 0);
     }
     format_pad(out, SCM_STRING(str), mincol, 1, padchar, TRUE);
 }
@@ -853,7 +854,7 @@ static void format_proc(ScmPort *out, ScmString *fmt, ScmObj args, int sharedp)
                         Scm_Error("'~*' format directive refers outside of argument list in %S", fmt);
                     }
                     argcnt = argindex;
-                    args = Scm_ListTail(oargs, argcnt);
+                    args = Scm_ListTail(oargs, argcnt, SCM_UNBOUND);
                     break;
                 }
             case 'v':; case 'V':;
@@ -1059,7 +1060,9 @@ static void vprintf_proc(ScmPort *out, const char *fmt, ScmObj args,
                     /* TODO: support right adjustment such as %-10s.
                        Currently we ignore minus sign and pad chars
                        on the right. */
-                    for (len = SCM_STRING_LENGTH(val); len < width; len++) {
+                    for (len = SCM_STRING_BODY_LENGTH(SCM_STRING_BODY(val));
+                         len < width;
+                         len++) {
                         Scm_PutcUnsafe(' ', out);
                     }
                     break;
