@@ -38,6 +38,10 @@
         ((sys-access "/sbin/pwd" |X_OK|) (get-command-output "/sbin/pwd"))
         (else (get-command-output "pwd"))))
 
+;; shorthand of normalizing pathname.  this doesn't do anything on
+;; unix, but on Windows the separator in PATHNAME is replaced.
+(define (n pathname) (sys-normalize-pathname pathname))
+
 ;;-------------------------------------------------------------------
 (test-section "environment")
 
@@ -45,7 +49,7 @@
        (sys-getenv "PATH"))
 
 (test* "getenv"
-       (let ((x (get-command-output "echo $NoSucHEniIRoNmenT")))
+       (let ((x (get-command-output "echo $NoSucHEnvIRoNmenT")))
          (if (string-null? x) #f x))
        (sys-getenv "NoSucHEniIRoNmenT"))
 
@@ -56,57 +60,66 @@
 (test-section "pathnames")
 
 (test* "basename" "ghi.jkl" (sys-basename "/abc/def/ghi.jkl"))
-(test* "dirname" "/abc/def" (sys-dirname "/abc/def/ghi.jkl"))
+(test* "dirname"  "/abc/def" (sys-dirname "/abc/def/ghi.jkl"))
 (test* "basename" "ghi.jkl" (sys-basename "/abc/def/ghi.jkl/"))
-(test* "dirname" "/abc/def" (sys-dirname "/abc/def/ghi.jkl/"))
+(test* "dirname"  "/abc/def" (sys-dirname "/abc/def/ghi.jkl/"))
 (test* "basename" "ghi.jkl" (sys-basename "/abc//def//ghi.jkl//"))
-(test* "dirname" "/abc//def" (sys-dirname "/abc//def//ghi.jkl//"))
+(test* "dirname"  "/abc//def" (sys-dirname "/abc//def//ghi.jkl//"))
 (test* "basename" "ghi.jkl" (sys-basename "ghi.jkl"))
 (test* "dirname" "." (sys-dirname "ghi.jkl"))
 
 (test* "basename" "" (sys-basename ""))
 (test* "dirname"  "." (sys-dirname ""))
 (test* "basename" "" (sys-basename "/"))
-(test* "dirname"  "/" (sys-dirname "/"))
+(test* "dirname"  (n "/") (sys-dirname "/"))
 (test* "basename" "" (sys-basename "//"))
-(test* "dirname"  "/" (sys-dirname "//"))
+(test* "dirname"  (n "/") (sys-dirname "//"))
 
 (test* "basename" ".." (sys-basename "../"))
 (test* "dirname"  "." (sys-dirname "../"))
 (test* "basename" ".." (sys-basename "../.."))
 (test* "dirname"  ".." (sys-dirname "../.."))
 
-(test* "normalize" (string-append (get-pwd-via-pwd) "/.")
+(test* "normalize" (n (string-append (get-pwd-via-pwd) "/."))
        (sys-normalize-pathname "." :absolute #t))
-(test* "normalize" (string-append (get-pwd-via-pwd) "/")
+(test* "normalize" (n (string-append (get-pwd-via-pwd) "/"))
        (sys-normalize-pathname "" :absolute #t))
-(test* "normalize" (string-append (get-command-output "echo $HOME") "/abc")
+(test* "normalize" (n (string-append (get-command-output "echo $HOME") "/abc"))
        (sys-normalize-pathname "~/abc" :expand #t))
-(test* "normalize" "/a/b/c/d/e"
+(test* "normalize" (n "/a/b/c/d/e")
        (sys-normalize-pathname "/a/b//.///c//d/./e"
                                :canonicalize #t))
-(test* "normalize" "/a/b/c/d/e/"
+(test* "normalize" (n "/a/b/c/d/e/")
        (sys-normalize-pathname "/a/b//.///c//d/./e/"
                                :canonicalize #t))
-(test* "normalize" "/a/b/c/d/e/"
+(test* "normalize" (n "/a/b/c/d/e/")
        (sys-normalize-pathname "/a/B//./../c/d/../../b//c/d/e/f/.."
                                :canonicalize #t))
+(test* "normalize" (n "/a/b/")
+       (sys-normalize-pathname "/a/b/c/d/../.."
+                               :canonicalize #t))
+(test* "normalize" (n "/c/d/")
+       (sys-normalize-pathname "/c/d/e/f/../../"
+                               :canonicalize #t))
+(test* "normalize" (n "/e/f/")
+       (sys-normalize-pathname "/e/f/g/h/../../."
+                               :canonicalize #t))
+
 (test* "normalize" ""
        (sys-normalize-pathname ""
                                :canonicalize #t))
-(test* "normalize" "a/../../.."
+(test* "normalize" (n "../..")
        (sys-normalize-pathname "a/b/c/../../../../.."
                                :canonicalize #t))
+(test* "normalize" (n "../../x/y")
+       (sys-normalize-pathname "a/b/c/../../../../../x/y"
+                               :canonicalize #t))
 
 ;;-------------------------------------------------------------------
 (test-section "filesystem")
 
 (sys-system "rm -rf test.dir >/dev/null")
 
-;; NB: on BSD-derived platforms, it seems that the change of
-;; permission bits does not propagate to the other process
-;; immediately (maybe due to the authentication module?).
-;; For the time being, we just put a delay before cheking.
 (test* "access" '(#f #f #f #f)
        (map (lambda (flag) (sys-access "test.dir" flag))
             (list |F_OK| |R_OK| |W_OK| |X_OK|)))
@@ -122,20 +135,22 @@
 (sys-system "touch test.dir/000")
 (sys-system "chmod 0000 test.dir/000")
 
-(sys-system "sleep 5")
+;; NB: access(2) causes problems on some platforms.
+;; Since its use is discouraged because of security concern
+;; (except F_OK), we do not test these any more.
 
-(test* "access" '(#t #t #t #t)
-       (map (lambda (flag) (sys-access "test.dir/777" flag))
-            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
-(test* "access" '(#t #t #f #t)
-       (map (lambda (flag) (sys-access "test.dir/500" flag))
-            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
-(test* "access" '(#t #t #f #f)
-       (map (lambda (flag) (sys-access "test.dir/400" flag))
-            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
-(test* "access" '(#t #f #f #f)
-       (map (lambda (flag) (sys-access "test.dir/000" flag))
-            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
+;(test* "access" '(#t #t #t #t)
+;       (map (lambda (flag) (sys-access "test.dir/777" flag))
+;            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
+;(test* "access" '(#t #t #f #t)
+;       (map (lambda (flag) (sys-access "test.dir/500" flag))
+;            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
+;(test* "access" '(#t #t #f #f)
+;       (map (lambda (flag) (sys-access "test.dir/400" flag))
+;            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
+;(test* "access" '(#t #f #f #f)
+;       (map (lambda (flag) (sys-access "test.dir/000" flag))
+;            (list |F_OK| |R_OK| |W_OK| |X_OK|)))
 
 (test* "sys-glob" '("test.dir/000" "test.dir/400" 
       "test.dir/500" "test.dir/777" "test.dir/999")
@@ -387,7 +402,7 @@
 ;;-------------------------------------------------------------------
 (test-section "select")
 
-(when (symbol-bound? 'sys-select)
+(when (global-variable-bound? 'gauche 'sys-select)
   (test* "fdset" '(3 #t #f #t #t #f)
          (let ((fdset (make <sys-fdset>)))
            (set! (sys-fdset-ref fdset (current-input-port)) #t)
@@ -440,7 +455,7 @@
                     (sys-waitpid pid)))))
              ))
          )
-  );; symbol-bound? sys-select
+  );; globla-variable-bound? sys-select
 
 ;;-------------------------------------------------------------------
 (test-section "signal handling")
@@ -493,7 +508,7 @@
             (sys-alarm 1)
             (read in)))))
 
-(when (symbol-bound? 'sys-select)
+(when (global-variable-bound? 'gauche 'sys-select)
   (test* "sigalrm6 (interrupting syscall - restart)" '(#t 0)
          (let1 r #f
            (with-signal-handlers
