@@ -314,13 +314,26 @@
 (use util.isomorph)
 (test-module 'util.isomorph)
 
+(define-class <isomorph-test> ()
+  ((a :init-keyword :a)
+   (b :init-keyword :b)))
+
+(define-method object-isomorphic? ((x <isomorph-test>)
+                                   (y <isomorph-test>)
+                                   context)
+  (and (isomorphic? (ref x 'a) (ref y 'a) context)
+       (isomorphic? (ref x 'b) (ref y 'b) context)))
+
 (define (make-data type)
-  (let* ((z (vector #f #f #f))
+  (let* ((z (vector #f #f #f #f))
          (x (circular-list "a" 'b 4 9845938427094857239485 #\z 8+5i z))
-         (y (circular-list "a" 'b 4 9845938427094857239485 #\z 8+5i z)))
+         (y (circular-list "a" 'b 4 9845938427094857239485 #\z 8+5i z))
+         (w (make <isomorph-test> :a z)))
     (vector-set! z 0 x)
     (vector-set! z 1 y)
-    (if type (vector-set! z 2 x) (vector-set! z 2 y))
+    (vector-set! z 2 w)
+    (slot-set! w 'b w)
+    (if type (vector-set! z 3 x) (vector-set! z 3 y))
     z))
 
 (test* "isomorphic?" #t
@@ -686,6 +699,166 @@
          (mod obj 2)
          (acc obj)))
 
+;;-----------------------------------------------
+(test-section "util.relation")
+(use util.relation)
+(test-module 'util.relation)
+(use gauche.sequence)
+
+(let ((r #f))
+  (test* "simple-relation" '<simple-relation>
+         (begin
+           (set! r (make <simple-relation>
+                     :columns '(a b c d e)
+                     :rows (list
+                            (vector 0 1 2 3 4)
+                            (vector 5 6 7 8 9)
+                            (vector 10 11 12 13 14)
+                            (vector 15 16 17 18 19))))
+           (class-name (class-of r))))
+
+  (test* "simple-relation (relation-ref)" '(0 5 10 15)
+         (map (cut relation-ref r <> 'a) r))
+
+  (test* "simple-relation (relation-accessor)" '(1 6 11 16)
+         (let1 ref (relation-accessor r)
+           (map (cut ref <> 'b) r)))
+
+  (test* "simple-relation (relation-column-getter)" '(2 7 12 17)
+         (let1 getter (relation-column-getter r 'c)
+           (map getter r)))
+
+  (test* "simple-relation (relation-set!)" '(100 105 110 115)
+         (begin
+           (for-each (lambda (row)
+                       (relation-set! r row 'a
+                                      (+ (relation-ref r row 'a) 100)))
+                     r)
+           (map (cut relation-ref r <> 'a) r)))
+
+  (test* "simple-relation (relation-modifier)" '(201 206 211 216)
+         (let ((set (relation-modifier r))
+               (ref (relation-accessor r)))
+           (for-each (lambda (row)
+                       (set row 'b (+ (ref row 'b) 200)))
+                     r)
+           (map (cut ref <> 'b) r)))
+
+  (test* "simple-relation (relation-column-setter)" '(304 309 314 319)
+         (let ((getter (relation-column-getter r 'e))
+               (setter (relation-column-setter r 'e)))
+           (for-each (lambda (row) (setter row (+ (getter row) 300))) r)
+           (map getter r)))
+
+  (test* "simple-relation (relation-fold)" 1264
+         (relation-fold r + 0 'a 'b))
+           
+  )
+
+(define-class <object-set-test> ()
+  ((a :init-value 0 :init-keyword :a)
+   (b :init-value 1 :init-keyword :b)
+   (c :init-value 2 :init-keyword :c)))
+
+(let ((r #f))
+  (test* "object-set-relation" '<object-set-relation>
+         (begin
+           (set! r (make <object-set-relation>
+                     :class <object-set-test>
+                     :rows (list
+                            (make <object-set-test> :a 0 :b 1 :c 2)
+                            (make <object-set-test> :a 3 :b 4 :c 5)
+                            (make <object-set-test> :a 6 :b 7 :c 8)
+                            (make <object-set-test> :a 9 :b 10 :c 11))))
+           (class-name (class-of r))))
+
+  (test* "object-set-relation (relation-ref)" '(0 3 6 9)
+         (map (cut relation-ref r <> 'a) r))
+
+  (test* "object-set-relation (relation-accessor)" '(1 4 7 10)
+         (let1 ref (relation-accessor r)
+           (map (cut ref <> 'b) r)))
+
+  (test* "object-set-relation (relation-column-getter)" '(2 5 8 11)
+         (let1 getter (relation-column-getter r 'c)
+           (map getter r)))
+
+  (test* "object-set-relation (relation-set!)" '(100 103 106 109)
+         (begin
+           (for-each (lambda (row)
+                       (relation-set! r row 'a
+                                      (+ (relation-ref r row 'a) 100)))
+                     r)
+           (map (cut relation-ref r <> 'a) r)))
+
+  (test* "object-set-relation (relation-modifier)" '(201 204 207 210)
+         (let ((set (relation-modifier r))
+               (ref (relation-accessor r)))
+           (for-each (lambda (row)
+                       (set row 'b (+ (ref row 'b) 200)))
+                     r)
+           (map (cut ref <> 'b) r)))
+
+  (test* "object-set-relation (relation-column-setter)" '(302 305 308 311)
+         (let ((getter (relation-column-getter r 'c))
+               (setter (relation-column-setter r 'c)))
+           (for-each (lambda (row) (setter row (+ (getter row) 300))) r)
+           (map getter r)))
+           
+  (test* "simple-relation (relation-fold)" 1644
+         (relation-fold r + 0 'a 'c))
+  )
+;;-----------------------------------------------
+(test-section "util.stream")
+(use util.stream)
+(test-module 'util.stream)
+
+(test* "stream?" #t
+       (stream? (stream-cons 'a stream-null)))
+
+(test* "stream-cons/car/cdr" '(1 1 1)
+       (letrec ((s (stream-cons 1 s)))
+         (list (stream-car s)
+               (stream-cadr s)
+               (stream-caddr s))))
+
+(test* "stream-delay" #t
+       (stream? (stream-delay (error "Boo"))))
+
+(test* "stream-unfoldn" '((0 2 4 6) (1 3 5 7))
+       (receive (s0 s1)
+           (stream-unfoldn (lambda (s)
+                             (values (+ s 2)
+                                     (list s)
+                                     (list (+ s 1))))
+                           0 2)
+         (map (lambda (s)
+                (list (stream-first s)
+                      (stream-second s)
+                      (stream-third s)
+                      (stream-fourth s)))
+              (list s0 s1))))
+
+(test* "stream-map/stream-ref" '(2 4 6 8)
+       (let1 s
+           (stream-map (cut * 2 <>)
+                       (stream-unfoldn (lambda (s) (values (+ s 1) (list s)))
+                                       0 1))
+         (map (cut stream-ref s <>) '(1 2 3 4))))
+
+(test* "stream-for-each/stream-iota" '(1 3 5 7)
+       (let1 r '()
+         (stream-for-each (lambda (x y)
+                            (push! r (+ x y)))
+                          (stream-iota 4)
+                          (stream-iota -1 1))
+         (reverse r)))
+
+(test* "stream-filter/take/drop" '(12 15 18 21)
+       (let1 s (stream-filter (lambda (x) (zero? (modulo x 3)))
+                              (stream-iota #f))
+         (stream->list (stream-take (stream-drop s 4) 4))))
+
 ;;-----------------------------------------------
 (test-section "util.toposort")
 (use util.toposort)
